# Imports from Coopr and PyUtilib
from coopr.pyomo import *
from coopr.core.plugin import *
from coopr.opt import *
import random
import copy


@plugin_factory
class MySolver(object):

    alias('random')

    # Declare that this is an IOptSolver plugin
    implements(IOptSolver)

    # Solve the specified problem and return
    # a SolverResults object
    def solve(self, instance, **kwds):
        print "Starting random heuristic"
        val, sol = self._random(instance)
        n = value(instance.N)
        # Setup results
        results = SolverResults()
        results.problem.name = instance.name
        results.problem.sense = ProblemSense.minimize
        results.problem.num_constraints = 1
        results.problem.num_variables = n
        results.problem.num_objectives = 1
        results.solver.status = SolverStatus.ok
        soln = results.solution.add()
        soln.value = val
        soln.status = SolutionStatus.feasible
        for j in xsequence(n):
            soln.variable[instance.y[j].name] = {"Value" : sol[j-1], "Id" : j}
        # Return results
        return results

    # Perform a random search
    def _random(self, instance):
        sol = [0]*instance.N.value
        for j in xrange(instance.P.value):
            sol[j] = 1
        # Generate 100 random solutions, and keep the best
        best = None
        best_sol = []
        for kk in xrange(100):
            random.shuffle(sol)
            # Compute value
            val=0.0
            for j in xsequence(instance.M.value):
                val += min([instance.d[i,j].value
                            for i in xsequence(instance.N.value)
                            if sol[i-1] == 1])
            if best is None or val < best:
                best=val
                best_sol=copy.copy(sol)
        return [best, best_sol]



import pyutilib.th as unittest
#from nose.tools import nottest
import coopr.pyomo

_campl_available = False
try:
    import coopr.pyomo.ampl.cAmpl as cAmpl
    _campl_available = True
except ImportError:
    pass

class MockFixedValue:
    value = 42
    def __init__(self, v = 42):
        self.value = v
    def is_fixed(self):
        return True

@unittest.skipUnless(_campl_available, "C AMPL module required")
class CAmplBasicTest(unittest.TestCase):
    def testNone(self):
        with self.assertRaises(ValueError) as cm:
            cAmpl.generate_ampl_repn(None)
    
    def testVar(self):
        testname = 'testname'

        var = coopr.pyomo.base.var._VarData(testname, None, None)
        var_ar = cAmpl.generate_ampl_repn(var)

        self.assertIsInstance(var_ar, coopr.pyomo.ampl.ampl_representation)

        self.assertEquals({testname:1.0}, var_ar._linear_terms_coef)

        self.assertEquals(1, len(var_ar._linear_terms_var))
        self.assertIsInstance(var_ar._linear_terms_var[testname], coopr.pyomo.base.var._VarData)

    def testExpressionBase(self):
        exp = coopr.pyomo.base.expr._ExpressionBase('name', 0, [])
        with self.assertRaises(ValueError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testSumExpression(self):
        exp = coopr.pyomo.base.expr._SumExpression()
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)

    def testProductExpression(self):
        x = coopr.pyomo.base.var.Var()
        y = coopr.pyomo.base.var.Var()
        exp = x * y
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertIs(exp_ar._nonlinear_expr, exp)
        self.assertTrue(exp_ar.is_nonlinear())

    def testProductExpressionZeroDiv(self):
        exp = coopr.pyomo.base.expr._ProductExpression()
        exp._numerator = [MockFixedValue(1)]
        exp._denominator = [MockFixedValue(0)]
        with self.assertRaises(ZeroDivisionError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testPowExpressionNoneArgs(self):
        exp = coopr.pyomo.base.expr._PowExpression([None, None])
        with self.assertRaises(ValueError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testPowExpressionConstants(self):
        v1 = MockFixedValue(2)
        v2 = MockFixedValue(3)

        exp = coopr.pyomo.base.expr._PowExpression([v1, v2])
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsNotNone(exp_ar)
        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)

        self.assertEquals(exp_ar._constant, 8)

    def testPowExpressionExp1(self):
        v1 = coopr.pyomo.base.var.Var()
        v2 = MockFixedValue(1)

        exp = coopr.pyomo.base.expr._PowExpression([v1, v2])
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsNotNone(exp_ar)
        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertEquals(exp_ar._linear_terms_var, cAmpl.generate_ampl_repn(v1)._linear_terms_var)

    def testPowExpressionExp0(self):
        v1 = coopr.pyomo.base.var.Var()
        v2 = MockFixedValue(0)

        exp = coopr.pyomo.base.expr._PowExpression([v1, v2])
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsNotNone(exp_ar)
        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertEquals(exp_ar._constant, 1)

    def testPowExpressionNonlinear(self):
        v1 = coopr.pyomo.base.var.Var(initialize=2)
        v2 = coopr.pyomo.base.var.Var(initialize=3)

        exp = coopr.pyomo.base.expr._PowExpression([v1, v2])
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsNotNone(exp_ar)
        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertIsInstance(exp_ar._nonlinear_expr, coopr.pyomo.base.expr._PowExpression)

    def testIntrinsicFunctionExpressionEmpty(self):
        exp = coopr.pyomo.base.expr._IntrinsicFunctionExpression('testname', 0, [], None)
        with self.assertRaises(AssertionError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testIntrinsicFunctionExpressionNoneArg(self):
        exp = coopr.pyomo.base.expr._IntrinsicFunctionExpression('sum', 1, [None], sum)
        with self.assertRaises(ValueError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testAbsFunctionExpression(self):
        exp = coopr.pyomo.base.expr._AbsExpression([2])
        with self.assertRaises(ValueError) as cm:
            exp_ar = cAmpl.generate_ampl_repn(exp)

    def testIntrinsicFunctionExpression(self):
        exp = coopr.pyomo.base.expr._IntrinsicFunctionExpression('sum', 1, [MockFixedValue()], sum)
        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsNotNone(exp_ar)
        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)

        self.assertEquals(type(exp), type(exp_ar._nonlinear_expr))
        self.assertEquals(exp.name, exp_ar._nonlinear_expr.name)
        self.assertEquals(0, len(exp_ar._nonlinear_vars))

    def testFixedValue(self):
        val = MockFixedValue()
        val_ar = cAmpl.generate_ampl_repn(val)

        self.assertIsInstance(val_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertEquals(MockFixedValue.value, val_ar._constant)

    def testCombinedProductSum(self):
        x = coopr.pyomo.base.var.Var()
        y = coopr.pyomo.base.var.Var()
        z = coopr.pyomo.base.var.Var()
        exp = x * y + z

        exp_ar = cAmpl.generate_ampl_repn(exp)

        self.assertIsInstance(exp_ar, coopr.pyomo.ampl.ampl_representation)
        self.assertTrue(exp_ar.is_nonlinear())

if __name__ == "__main__":
    unittest.main(verbosity=2)



from coopr.pyomo import *

model = AbstractModel()
model.X = Var()

if model.X >= 10.0:
    pass
if value(model.X) >= 10.0:
    pass
if model.X >= 10.0:
    pass

if model.X >= 10.0:
    if model.X >= 10.0:
        pass



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['SparseIndexedComponent', 'ActiveSparseIndexedComponent']

from six import iterkeys, itervalues, iteritems
import pyutilib.misc
from coopr.pyomo.base.component import Component

UnindexedComponent_set = set([None])


def normalize_index(index):
    """
    Flatten a component index.  If it has length 1, then 
    return just the element.  If it has length > 1, then
    return a tuple.
    """
    ndx = pyutilib.misc.flatten(index)
    if type(ndx) is list:
        if len(ndx) == 1:
            ndx = ndx[0]
        else:
            ndx = tuple(ndx)
    return ndx


class SparseIndexedComponent(Component):
    """
    This is the base class for all indexed modeling components.
    This class stores a dictionary, self._data, that maps indices
    to component data objects.  The object self._index defines valid
    keys for this dictionary, and the dictionary keys may be a
    strict subset.  

    The standard access and iteration methods iterate over the the
    keys of self._data.  This class supports a concept of a default
    component data value.  When enabled, the default does not
    change the access and iteration methods.

    Constructor arguments:
        ctype       The class type for the derived subclass
        doc         A text string describing this component

    Private class attributes:
        _data               A dictionary from the index set to 
                                component data objects
        _index              The set of valid indices
        _implicit_subsets   A temporary data element that stores
                                sets that are transfered to the model
    """

    #
    # If an index is supplied for which there is not a _data entry 
    # (specifically, in a get call), then this flag determines whether 
    # a check is performed to see if the input index is in the 
    # index set _index. This is extremely expensive, and so this flag
    # is provided to disable that feature globally.
    #
    _DEFAULT_INDEX_CHECKING_ENABLED = True

    def __init__(self, *args, **kwds):
        from coopr.pyomo.base.sets import process_setarg
        #
        Component.__init__(self, **kwds)
        #
        self._data = {}
        #
        if len(args) == 0:
            #
            # If no indexing sets are provided, generate a dummy index
            #
            self._implicit_subsets = None
            self._index = UnindexedComponent_set
            self._data = {None: None}
        elif len(args) == 1:
            #
            # If a single indexing set is provided, just process it.
            #
            self._implicit_subsets = None
            self._index = process_setarg(args[0])
        else:
            #
            # If multiple indexing sets are provided, process them all,
            # and store the cross-product of these sets.  The individual
            # sets need to stored in the Pyomo model, so the
            # _implicit_subsets class data is used for this temporary
            # storage.
            # 
            # Example:  Pyomo allows things like 
            # "Param([1,2,3], range(100), initialize=0)".  This
            # needs to create *3* sets: two SetOf components and then
            # the SetProduct.  That means that the component needs to
            # hold on to the implicit SetOf objects until the component
            # is assigned to a model (where the implicit subsets can be
            # "transferred" to the model).
            #
            tmp = [process_setarg(x) for x in args]
            self._implicit_subsets = tmp
            self._index = tmp[0].cross(*tmp[1:])

    def clear(self):
        """Clear the data in this component"""
        if UnindexedComponent_set != self._index:
            self._data = {}
        else:
            raise NotImplementedError(
                "Derived scalar component %s failed to define clear().\n"
                "\tPlease report this to the Pyomo developers"
                % (self.__class__.__name__,))

    def index_set(self):
        """Return the index set"""
        return self._index

    def is_indexed(self):
        """Return true if this component is indexed"""
        return UnindexedComponent_set != self._index

    def dim(self):
        """Return the dimension of the index"""
        if UnindexedComponent_set != self._index:
            return self._index.dimen
        else:
            return 0

    def __len__(self):
        """
        Return the number of component data objects stored by this
        component.  
        """
        return len(self._data)

    def __contains__(self, ndx):
        """Return true if the index is in the dictionary"""
        return ndx in self._data

    def __iter__(self):
        """Iterate over the keys in the dictionary"""
        if len(self._data) == len(self._index):
            #
            # If the data is dense then return the index iterator.
            #
            return self._index.__iter__()
        elif not hasattr(self._index, 'ordered') or not self._index.ordered:
            #
            # If the index set is not ordered, then return the 
            # data iterator.  This is in an arbitrary order, which is 
            # fine because the data is unordered.
            #
            return self._data.__iter__()
        else:
            #
            # Test each element of a sparse data with an ordered index set
            # in order.  This is potentially *slow*: if the component is in fact
            # very sparse, we could be iterating over a huge (dense)
            # index in order to sort a small number of indices.
            # However, this provides a consistent ordering that the user
            # expects. 
            #
            def _sparse_iter_gen(self):
                for idx in self._index.__iter__():
                    if idx in self._data:
                        yield idx
            return _sparse_iter_gen(self)

    def keys(self):
        """Return a list of keys in the dictionary"""
        return [ x for x in self ]

    def values(self):
        """Return a list of the component data objects in the dictionary"""
        return [ self[x] for x in self ]

    def items(self):
        """Return a list (index,data) tuples from the dictionary"""
        return [ (x, self[x]) for x in self ]

    def iterkeys(self):
        """Return an iterator of the keys in the dictionary"""
        return self.__iter__()

    def itervalues(self):
        """Return an iterator of the component data objects in the dictionary"""
        for key in self:
            yield self[key]
    
    def iteritems(self):
        """Return an iterator of (index,data) tuples from the dictionary"""
        for key in self:
            yield key, self[key]

    def __getitem__(self, ndx):
        """
        This method returns the data corresponding to the given index.
        """
        if ndx in self._data:
            # Return the data from the dictionary
            if ndx is None:
                return self
            else:
                return self._data[ndx]
        elif not self._constructed:
            # Generate an error because the component is not constructed
            if ndx is None:
                idx_str = ''
            elif ndx.__class__ is tuple:
                idx_str = "[" + ",".join(str(i) for i in ndx) + "]"
            else:
                idx_str = "[" + str(ndx) + "]"
            raise ValueError(
                "Error retrieving component %s%s: The component has "
                "not been constructed." % ( self.cname(True), idx_str,) )
        elif not SparseIndexedComponent._DEFAULT_INDEX_CHECKING_ENABLED:
            # Return the default value if the global flag dictates that
            return self._default(ndx)            
        elif ndx in self._index:
            # After checking that the index is value, return the default value
            # This check is expensive!
            return self._default(ndx)
        else:
            # Now we normalize the index and check again.  Usually,
            # indices will be normalized, so this operation is deferred.
            ndx = normalize_index(ndx)
            if ndx in self._data:
                # Note that ndx != None at this point
                return self._data[ndx]
            elif not SparseIndexedComponent._DEFAULT_INDEX_CHECKING_ENABLED:
                return self._default(ndx)            
            elif ndx in self._index:
                return self._default(ndx)
        #
        # Generate different errors, depending on the state of the index.
        #
        if not self.is_indexed():
            msg = "Error accessing indexed component: " \
                  "Cannot treat the scalar component '%s' as an array" \
                  % ( self.cname(True), )
        else:
            msg = "Error accessing indexed component: " \
                  "Index '%s' is not valid for array component '%s'" \
                  % ( ndx, self.cname(True), )
        raise KeyError(msg)

    def _default(self, index):
        """Returns the default component data value"""
        raise NotImplementedError(
            "Derived component %s failed to define _default().\n"
            "\tPlease report this to the Pyomo developers"
            % (self.__class__.__name__,))

    def set_value(self, value):
        """Set the value of a scalar component."""
        if UnindexedComponent_set != self._index:
            raise ValueError(
                "Cannot set the value for the indexed component '%s' "
                "without specifying an index value.\n"
                "\tFor example, model.%s[i] = value"
                % (self.name, self.name))
        else:
            raise NotImplementedError(
                "Derived component %s failed to define set_value() "
                "for scalar instances.\n"
                "\tPlease report this to the Pyomo developers"
                % (self.__class__.__name__,))

    def id_index_map(self):
        """
        Return an dictionary id->index for
        all ComponentData instances.
        """
        result = {}
        for index, component_data in iteritems(self):
            result[id(component_data)] = index
        return result


class ActiveSparseIndexedComponent(SparseIndexedComponent):
    """
    This is the base class for all indexed modeling components
    whose data members are subclasses of ActiveComponentData, e.g.,
    can be activated or deactivated.

    The activate and deactivate methods activate both the 
    component as well as all component data values.
    """

    def __init__(self, *args, **kwds):
        SparseIndexedComponent.__init__(self, *args, **kwds)

    def activate(self):
        """Set the active attribute to True"""
        Component.activate(self)
        for component_data in itervalues(self):
            component_data._active = True

    def deactivate(self):
        """Set the active attribute to False"""
        Component.deactivate(self)
        for component_data in itervalues(self):
            component_data._active = False




#import argparse
import coopr.misc.coopr_parser
from coopr.pyomo import TransformationFactory
import textwrap


def main_exec(options):
    import coopr.environ
    wrapper = textwrap.TextWrapper()
    wrapper.initial_indent = '      '
    wrapper.subsequent_indent = '      '
    print("")
    for xform in sorted(TransformationFactory.services()):
        print("  "+xform)
        print(wrapper.fill(TransformationFactory.doc(xform)))


#
# Add a subparser for the check command
#
coopr.misc.coopr_parser.add_subparser('transformations',
        func=main_exec, 
        help='List the available model transformations.',
        description='This coopr subcommand is used to list the available model transformations.'
        )



import ast

from coopr.pyomo.plugins.check.model import ModelTrackerHook
from coopr.pyomo.plugins.check.checker import IterativeTreeChecker
from coopr.pyomo.plugins.check.checkers.pyomo._rulebase import _ModelRuleChecker


class ModelShadowing(IterativeTreeChecker):

    ModelTrackerHook()

    def checkerDoc(self):
        return """\
        It is generally considered poor practice to "shadow", or reuse,
        the name of your model variable in a rule. In your rule definitions,
        consider changing the name of the model argument.
        """

    def check(self, runner, script, info):
        if isinstance(info, ast.FunctionDef):
            for arg in info.args.args:
                if isinstance(arg, ast.Name):
                    if arg.id in script.modelVars:
                        self.problem("Function {0} may shadow model variable {1}".format(info.name, arg.id), lineno=info.lineno)


class ModelAccess(IterativeTreeChecker):

    ModelTrackerHook()

    def checkerDoc(self):
        return """\
        Within model rules, you should access the instance of the model that
        is passed in to the function, rather than the global model instance.
        For example:
            def rule(m, i):
                return m.x[i] >= 10.0 # not model.x[i]
        """

    def check(self, runner, script, info):
        if isinstance(info, ast.FunctionDef):
            attrNodes = [x for x in list(ast.walk(info)) if isinstance(x, ast.Attribute)]
            for attrNode in attrNodes:
                if attrNode.value.id in script.modelVars:
                    self.problem("Expression {0}.{1} may access model outside function scope".format(attrNode.value.id, attrNode.attr), lineno=attrNode.lineno)


class ModelArgument(_ModelRuleChecker):

    def checkerDoc(self):
        return """\
        Model rule functions must have the model as the first argument in
        the function definition. For example, change:
            def con_rule(i, model): # ...
        To:
            def con_rule(model, i): # ...
        """

    def checkBody(self, funcdef):
        for bodyNode in funcdef.body:
            for node in ast.walk(bodyNode):
                if isinstance(node, ast.Attribute):
                    if isinstance(node.value, ast.Name):
                        if node.value.id != funcdef.args.args[0].id:
                            self.problem("Model argument {0} is not first in rule argument list".format(node.value.id), lineno=funcdef.lineno)


class NoneReturn(_ModelRuleChecker):

    def checkerDoc(self):
        return """\
        Model rule functions may not return None.
        """

    def checkBody(self, funcdef):
        """Look for statements of the format 'return None'"""
        for node in ast.walk(funcdef):
            if isinstance(node, ast.Return):
                if isinstance(node.value, ast.Name):
                    if node.value.id == 'None':
                        self.problem("Cannot return None from model rule {0}".format(funcdef.name), lineno=funcdef.lineno)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import coopr.pyomo.plugins.transform.relax_integrality
import coopr.pyomo.plugins.transform.eliminate_fixed_vars
import coopr.pyomo.plugins.transform.standard_form
import coopr.pyomo.plugins.transform.equality_transform
import coopr.pyomo.plugins.transform.nonnegative_transform
import coopr.pyomo.plugins.transform.dual_transformation
import coopr.pyomo.plugins.transform.util



from inspect import isroutine

from coopr.core.plugin import alias

from coopr.pyomo import *
from coopr.pyomo.base.var import SimpleVar, _VarData
from coopr.pyomo.base.expr import _SumExpression, _ProductExpression, \
     _AbsExpression, _PowExpression
from coopr.pyomo.base.expression import _ExpressionData
from coopr.pyomo.base.numvalue import create_name
from coopr.pyomo.plugins.transform.util import partial
from coopr.pyomo.plugins.transform.hierarchy import IsomorphicTransformation
from coopr.pyomo.plugins.transform.util import collectAbstractComponents


class NonNegativeTransformation(IsomorphicTransformation):
    """
    Creates a new, equivalent model by forcing all variables to lie in
    the nonnegative orthant by introducing auxiliary variables.
    """

    alias("nonnegative_vars", doc="Create an equivalent model in which all variables lie in the nonnegative orthant.")

    def __init__(self, **kwds):
        kwds["name"] = kwds.pop("name", "vars")
        super(NonNegativeTransformation, self).__init__(**kwds)

        self.realSets = (Reals, PositiveReals, NonNegativeReals, NegativeReals,
                         NonPositiveReals, PercentFraction, RealSet)
        # Intentionally leave out Binary, Boolean, BinarySet, and BooleanSet;
        # we check for those explicitly
        self.discreteSets = (IntegerSet, Integers, PositiveIntegers,
                             NonPositiveIntegers, NegativeIntegers,
                             NonNegativeIntegers)


    def apply(self, model, **kwds):
        """
        Force all variables to lie in the nonnegative orthant.

        Required arguments:
            model       The model to transform.

        Optional keyword arguments:
          pos_suffix    The suffix applied to the 'positive' component of
                            converted variables. Default is '_plus'.
          neg_suffix    The suffix applied to the 'positive' component of
                            converted variables. Default is '_minus'.
        """
        #
        # Optional naming schemes
        #
        pos_suffix = kwds.pop("pos_suffix", "_plus")
        neg_suffix = kwds.pop("neg_suffix", "_minus")
        #
        # We first perform an abstract problem transformation. Then, if model
        # data is available, we instantiate the new model. If not, we construct
        # a mapping that can later be used to populate the new model.
        #
        nonneg = model.clone()
        components = collectAbstractComponents(nonneg)

        # Map from variable base names to a {index, rule} map
        constraint_rules = {}

        # Map from variable base names to a rule defining the domains for that
        # variable
        domain_rules = {}

        # Map from variable base names to its set of indices
        var_indices = {}

        # Map from fully qualified variable names to replacement expressions.
        # For now, it is actually a map from a variable name to a closure that
        # must later be evaulated with a model containing the replacement
        # variables.
        var_map = {}

        #
        # Get the constraints that enforce the bounds and domains of each
        # variable
        #
        for var_name in components["Var"]:
            var = nonneg.__getattribute__(var_name)

            # Individual bounds and domains
            orig_bounds = {}
            orig_domain = {}

            # New indices
            indices = set()

            # Map from constraint names to a constraint rule.
            constraints = {}

            # Map from variable indices to a domain
            domains = {}

            for ndx in var:
                # Fully qualified variable name
                vname = create_name(str(var_name), ndx)

                # We convert each index to a string to avoid difficult issues
                # regarding appending a suffix to tuples.
                #
                # If the index is None, this casts the index to a string,
                # which doesn't match up with how Pyomo treats None indices
                # internally. Replace with "" to be consistent.
                if ndx is None:
                    v_ndx = ""
                else:
                    v_ndx = str(ndx)

                # Get the variable bounds
                lb = var[ndx].lb
                ub = var[ndx].ub
                if lb is not None:
                    lb = value(lb)
                if ub is not None:
                    ub = value(ub)
                orig_bounds[ndx] = (lb, ub)

                # Get the variable domain
                if var[ndx].domain is not None:
                    orig_domain[ndx] = var[ndx].domain
                else:
                    orig_domain[ndx] = var.domain

                # Determine the replacement expression. Either a new single
                # variable with the same attributes, or a sum of two new
                # variables.
                #
                # If both the bounds and domain allow for negative values,
                # replace the variable with the sum of nonnegative ones.

                bounds_neg = (orig_bounds[ndx] == (None, None) or
                              orig_bounds[ndx][0] is None or
                              orig_bounds[ndx][0] < 0)
                domain_neg = (orig_domain[ndx] is None or
                              orig_domain[ndx].bounds()[0] is None or
                              orig_domain[ndx].bounds()[0] < 0)
                if bounds_neg and domain_neg:
                    # Make two new variables.
                    posVarSuffix = "%s%s" % (v_ndx, pos_suffix)
                    negVarSuffix = "%s%s" % (v_ndx, neg_suffix)

                    new_indices = (posVarSuffix, negVarSuffix)

                    # Replace the original variable with a sum expression
                    expr_dict = {posVarSuffix: 1, negVarSuffix: -1}
                else:
                    # Add the new index. Lie if is 'None', since Pyomo treats
                    # 'None' specially as a key.
                    #
                    # More lies: don't let a blank index exist. Replace it with
                    # '_'. I don't actually have a justification for this other
                    # than that allowing "" as a key will eventually almost
                    # certainly lead to a strange bug.
                    if v_ndx is None:
                        t_ndx = "None"
                    elif v_ndx == "":
                        t_ndx = "_"
                    else:
                        t_ndx = v_ndx

                    new_indices = (t_ndx,)

                    # Replace the original variable with a sum expression
                    expr_dict = {t_ndx: 1}

                # Add the new indices
                for x in new_indices:
                    indices.add(x)

                # Replace the original variable with an expression
                var_map[vname] = partial(self.sumRule,
                                         var_name,
                                         expr_dict)

                # Enforce bounds as constraints
                if orig_bounds[ndx] != (None, None):
                    cname = "%s_%s" % (vname, "bounds")
                    tmp = orig_bounds[ndx]
                    constraints[cname] = partial(
                        self.boundsConstraintRule,
                        tmp[0],
                        tmp[1],
                        var_name,
                        expr_dict)

                # Enforce the bounds of the domain as constraints
                if orig_domain[ndx] != None:
                    cname = "%s_%s" % (vname, "domain_bounds")
                    tmp = orig_domain[ndx].bounds()
                    constraints[cname] = partial(
                        self.boundsConstraintRule,
                        tmp[0],
                        tmp[1],
                        var_name,
                        expr_dict)

                # Domain will either be NonNegativeReals, NonNegativeIntegers,
                # or Binary. We consider Binary because some solvers may
                # optimize over binary variables.
                if isinstance(orig_domain[ndx], RealSet):
                    for x in new_indices:
                        domains[x] = NonNegativeReals
                elif isinstance(orig_domain[ndx], IntegerSet):
                    for x in new_indices:
                        domains[x] = NonNegativeIntegers
                elif isinstance(orig_domain[ndx], BooleanSet):
                    for x in new_indices:
                        domains[x] = Binary
                else:
                    print ("Warning: domain '%s' not recognized, " + \
                           "defaulting to 'Reals'") % (str(var.domain))
                    for x in new_indices:
                        domains[x] = Reals

            constraint_rules[var_name] = constraints
            domain_rules[var_name] = partial(self.exprMapRule, domains)
            var_indices[var_name] = indices

        # Remove all existing variables.
        toRemove = []
        for (attr_name, attr) in nonneg.__dict__.items():
            if isinstance(attr, Var):
                toRemove.append(attr_name)
        for attr_name in toRemove:
            nonneg.__delattr__(attr_name)

        # Add the sets defining the variables, then the variables
        for (k, v) in var_indices.items():
            sname = "%s_indices" % k
            nonneg.__setattr__(sname, Set(initialize=v))
            nonneg.__setattr__(k, Var(nonneg.__getattribute__(sname),
                                      domain = domain_rules[k],
                                      bounds = (0, None)))

        # Construct the model to get the variables and their indices
        # recognized in the model
        nonneg = nonneg.create()

        # Safe to evaluate the modifiedVars mapping
        for var in var_map:
            var_map[var] = var_map[var](nonneg)

        # Map from constraint base names to maps from indices to expressions
        constraintExprs = {}

        #
        # Convert all modified variables in all constraints in the original
        # problem
        #
        for conName in components["Constraint"]:
            con = nonneg.__getattribute__(conName)

            # Map from constraint indices to a corrected expression
            exprMap = {}

            for (ndx, cdata) in con._data.items():
                lower = self._walk_expr(cdata.lower, var_map)
                body  = self._walk_expr(cdata.body,  var_map)
                upper = self._walk_expr(cdata.upper, var_map)

                # Lie if ndx is None. Pyomo treats 'None' indices specially.
                if ndx is None:
                    ndx = "None"

                # Cast indices to strings, otherwise tuples ruin everything
                exprMap[str(ndx)] = (lower, body, upper)

            # Add to list of expression maps
            constraintExprs[conName] = exprMap

        # Map from constraint base names to maps from indices to expressions
        objectiveExprs = {}

        #
        # Convert all modified variables in all objectives in the original
        # problem
        #
        for objName in components["Objective"]:
            obj = nonneg.__getattribute__(objName)

            # Map from objective indices to a corrected expression
            exprMap = {}

            for (ndx, odata) in obj._data.items():
                exprMap[ndx] = self._walk_expr(odata.expr, var_map)

            # Add to list of expression maps
            objectiveExprs[objName] = exprMap


        # Make the modified original constraints
        for (conName, ruleMap) in constraintExprs.items():
            # Make the set of indices
            sname = conName + "_indices"
            _set = Set(initialize=ruleMap.keys())
            nonneg.__setattr__(sname, _set)
            _set.construct()

            # Define the constraint
            _con = Constraint( nonneg.__getattribute__(sname),
                               rule=partial(self.exprMapRule, ruleMap) )
            nonneg.__setattr__(conName, _con)
            _con.construct()

        # Make the bounds constraints
        for (varName, ruleMap) in constraint_rules.items():
            conName = varName + "_constraints"
            # Make the set of indices
            sname = conName + "_indices"
            _set = Set(initialize=ruleMap.keys())
            nonneg.__setattr__(sname, _set)
            _set.construct()

            # Define the constraint
            _con = Constraint(nonneg.__getattribute__(sname),
                              rule=partial(self.delayedExprMapRule, ruleMap))
            nonneg.__setattr__(conName, _con)
            _con.construct()

        # Make the objectives
        for (objName, ruleMap) in objectiveExprs.items():
            # Make the set of indices
            sname = objName + "_indices"
            _set = Set(initialize=ruleMap.keys())
            nonneg.__setattr__(sname, _set)
            _set.construct()

            # Define the constraint
            _obj = Objective(nonneg.__getattribute__(sname),
                             rule=partial(self.exprMapRule, ruleMap))
            nonneg.__setattr__(objName, _obj)
            _obj.construct()

        nonneg.preprocess()
        return nonneg
        


    @staticmethod
    def _walk_expr(expr, varMap):
        """
        Walks an expression tree, making the replacements defined in varMap
        """

        # Attempt to replace a simple variable
        if isinstance(expr, SimpleVar):
            if expr.name in varMap:
                return varMap[expr.name]
            else:
                return expr

        # Attempt to replace an indexed variable
        if isinstance(expr, _VarData):
            if expr.name in varMap:
                return varMap[expr.name]
            else:
                return expr

        # Iterate through the numerator and denominator of a product term
        if isinstance(expr, _ProductExpression):
            i = 0
            while i < len(expr._numerator):
                expr._numerator[i] = NonNegativeTransformation._walk_expr(
                    expr._numerator[i],
                    varMap)
                i += 1

            i = 0
            while i < len(expr._denominator):
                expr._denominator[i] = NonNegativeTransformation._walk_expr(
                    expr.denominator[i],
                    varMap)
                i += 1

        # Iterate through the terms in a sum, absolute value term, or power
        # term
        if isinstance(expr, (_SumExpression, _AbsExpression, _PowExpression,
                             _ExpressionData)):
            i = 0
            while i < len(expr._args):
                expr._args[i] = NonNegativeTransformation._walk_expr(
                    expr._args[i],
                    varMap)
                i += 1

        return expr

    @staticmethod
    def boundsConstraintRule(lb, ub, attr, vars, model):
        """
        Produces 'lb < x^+ - x^- < ub' style constraints. Designed to
        be made a closer through functools.partial, across lb, ub, attr,
        and vars. vars is a {varname: coefficient} dictionary. attr is the
        base variable name; that is, X[1] would be referenced by

          model.__getattribute__('X')[1]

        and so attr='X', and 1 is a key of vars.

        """
        return (lb,
                sum(c * model.__getattribute__(attr)[v] \
                    for (v,c) in vars.items()),
                ub)

    @staticmethod
    def noConstraint(*args):
        return None

    @staticmethod
    def sumRule(attr, vars, model):
        """
        Returns a sum expression.
        """
        return sum(c*model.__getattribute__(attr)[v] for (v, c) in vars.items())

    @staticmethod
    def exprMapRule(ruleMap, model, ndx=None):
        """ Rule intended to return expressions from a lookup table """
        return ruleMap[ndx]

    @staticmethod
    def delayedExprMapRule(ruleMap, model, ndx=None):
        """
        Rule intended to return expressions from a lookup table. Each entry
        in the lookup table is a functor that needs to be evaluated before
        returning.
        """
        return ruleMap[ndx](model)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

from coopr.core.plugin import alias
from coopr.pyomo import Binary, value, as_numeric
from coopr.pyomo.base import Transformation, Var, Constraint, ConstraintList, Block, RangeSet
from coopr.pyomo.base.expr import _ProductExpression, _PowExpression
from coopr.pyomo.base.var import _VarData

from six import iteritems

import logging
logger = logging.getLogger(__name__)

class RadixLinearization(Transformation):
    """
    This plugin generates linear relaxations of bilinear problems using
    the multiparametric disaggregation technique of Kolodziej, Castro,
    and Grossmann.  See:

    Scott Kolodziej, Pedro M. Castro, and Ignacio E. Grossmann. "Global
       optimization of bilinear programs with a multiparametric
       disaggregation technique."  J.Glob.Optim 57 pp.1039-1063. 2013. 
       (DOI 10.1007/s10898-012-0022-1)
    """

    alias( "radix_linearization", 
           doc="Linearize bilinear and quadratic terms through "
           "radix discretization (multiparametric disaggregation)" )

    def apply(self, model, **kwds):
        precision = kwds.pop('precision',8)
        user_discretize = kwds.pop('discretize',set())
        verbose = kwds.pop('verbose',False)

        M = model.clone()

        # TODO: if discretize is not empty, we must translate those
        # components over to the components on the cloned instance
        _discretize = {}
        if user_discretize:
            for _var in user_discretize:
                _v = M.find_component(_var.cname(True))
                if _v.component() is _v:
                    for _vv in _v.itervalues():
                        _discretize.setdefault(id(_vv), len(_discretize))
                else:
                    _discretize.setdefault(id(_v), len(_discretize))

        # Iterate over all Constraints and identify the bilinear and
        # quadratic terms
        bilinear_terms = []
        quadratic_terms = []
        for constraint in M.active_subcomponents(Constraint).itervalues():
            for cname, c in constraint._data.iteritems():
                if c.body.polynomial_degree() != 2:
                    continue
                self._collect_bilinear(c.body, bilinear_terms, quadratic_terms)

        # We want to find the (minimum?) number of variables to
        # discretize so that we cover all the bilinearities -- without
        # discretizing both sides of any single bilinear expression.
        # First step: figure out how many expressions each term appears
        # in
        _counts = {}
        for q in quadratic_terms:
            if not q[1].is_continuous():
                continue
            _id = id(q[1])
            if _id not in _counts:
                _counts[_id] = (q[1], set())
            _counts[_id][1].add(_id)            
        for bi in bilinear_terms:
            for i in (0,1):
                if not bi[i+1].is_continuous():
                    continue
                _id = id(bi[i+1])
                if _id not in _counts:
                    _counts[_id] = (bi[i+1], set())
                _counts[_id][1].add(id(bi[2-i]))

        _tmp_counts = dict(_counts)
        # First, remove the variables that the user wants to have discretized
        for _id in _discretize:
            for _i in _tmp_counts[_id][1]:
                if _i == _id:
                    continue
                _tmp_counts[_i][1].remove(_id)
            del _tmp_counts[_id]
        # All quadratic terms must be discretized (?)
        #for q in quadratic_terms:
        #    _id = id(q[1])
        #    if _id not in _tmp_counts:
        #        continue
        #    _discretize.setdefault(_id, len(_discretize))
        #    for _i in _tmp_counts[_id][1]:
        #        if _i == _id:
        #            continue
        #        _tmp_counts[_i][1].remove(_id)
        #    del _tmp_counts[_id]
            
        # Now pick a (minimal) subset of the terms in bilinear expressions
        while _tmp_counts:
            _ct, _id = max( (len(_tmp_counts[i][1]), i) for i in _tmp_counts )
            if not _ct:
                break
            _discretize.setdefault(_id, len(_discretize))
            for _i in list(_tmp_counts[_id][1]):
                if _i == _id:
                    continue
                _tmp_counts[_i][1].remove(_id)
            del _tmp_counts[_id]

        #
        # Discretize things
        #

        # Define a block (namespace) for holding the disaggregated
        # variables and new constraints
        if False: # Set to true when the LP writer is fixed
            M._radix_linearization = Block()
            _block = M._radix_linearization
        else:
            _block = M
        _block.DISCRETIZATION = RangeSet(precision)
        _block.DISCRETIZED_VARIABLES = RangeSet(0, len(_discretize)-1)
        _block.z = Var( _block.DISCRETIZED_VARIABLES, _block.DISCRETIZATION, 
                         within=Binary )
        _block.dv = Var( _block.DISCRETIZED_VARIABLES, 
                         bounds=(0,2**-precision) )
        
        # Actually discretize the terms we have marked for discretization
        for _id, _idx in iteritems(_discretize):
            if verbose:
                logger.info("Discretizing variable %s as %s" % 
                            (_counts[_id][0].cname(True), _idx))
            self._discretize_variable(_block, _counts[_id][0], _idx)

        _known_bilinear = {}
        # For each quadratic term, if it hasn't been discretized /
        # generated, do so, and remember the resulting W term for later
        # use...
        #for _expr, _x1 in quadratic_terms:
        #    self._discretize_term( _expr, _x1, _x1, 
        #                           _block, _discretize, _known_bilinear )
        # For each bilinear term, if it hasn't been discretized /
        # generated, do so, and remember the resulting W term for later
        # use...
        for _expr, _x1, _x2 in bilinear_terms:
            self._discretize_term( _expr, _x1, _x2, 
                                   _block, _discretize, _known_bilinear )

        # Return the discretized instance!
        return M


    def _discretize_variable(self, b, v, idx):
        _lb, _ub = v.bounds
        if _lb is None or _ub is None:
            raise RuntimeError("Couldn't discretize variable %s: missing "
                               "finite lower/upper bounds." % (v.cname(True)))
        _c = Constraint( 
            expr= v == _lb + (_ub-_lb) * ( b.dv[idx] + 
                sum(b.z[idx,k] * 2**-k for k in b.DISCRETIZATION) ) )
        b.add_component("c_discr_v%s" % idx, _c)


    def _discretize_term(self, _expr, _x1, _x2, _block, _discretize, _known_bilinear):
        if id(_x1) in _discretize:
            _v = _x1
            _u = _x2
        elif id(_x2) in _discretize:
            _u = _x1
            _v = _x2
        else:
            raise RuntimeError("Couldn't identify discretized variable "
                               "for expression '%s'!" % _expr)
        _id = (id(_v), id(_u))
        if _id not in _known_bilinear:
            _known_bilinear[_id] = self._discretize_bilinear(
                _block, _v, _discretize[id(_v)], _u, len(_known_bilinear))
        # _expr should be a "simple" product expression; substitute
        # in the bilinear "W" term for the raw bilinear terms
        _expr._numerator = [ _known_bilinear[_id] ]


    def _discretize_bilinear(self, b, v, v_idx, u, u_idx):
        _z = b.z
        _dv = b.dv[v_idx]
        _u = Var(b.DISCRETIZATION, within=u.domain, bounds=u.bounds)
        logger.info("Discretizing (v=%s)*(u=%s) as u%s_v%s" % (
                v.cname(True), u.cname(True), u_idx, v_idx ))
        b.add_component( "u%s_v%s" % (u_idx, v_idx), _u)
        _lb, _ub = u.bounds
        if _lb is None or _ub is None:
             raise RuntimeError("Couldn't relax variable %s: missing "
                               "finite lower/upper bounds." % (u.cname(True)))
        _c = ConstraintList(noruleinit=True)
        b.add_component( "c_disaggregate_u%s_v%s" % (u_idx, v_idx), _c )
        for k in b.DISCRETIZATION:
            # _lb * z[v_idx,k] <= _u[k] <= _ub * z[v_idx,k]
            _c.add(expr= _lb*_z[v_idx,k] <= _u[k] )
            _c.add(expr= _u[k] <= _ub*_z[v_idx,k] )
            # _lb * (1-z[v_idx,k]) <= u - _u[k] <= _ub * (1-z[v_idx,k])
            _c.add(expr= _lb * (1-_z[v_idx,k]) <= u - _u[k] )
            _c.add(expr= u - _u[k] <= _ub * (1-_z[v_idx,k]))

        _v_lb, _v_ub = v.bounds
        _bnd_rng = (_v_lb*_lb, _v_lb*_ub, _v_ub*_lb, _v_ub*_ub)
        _w = Var(bounds=(min(_bnd_rng), max(_bnd_rng)))
        b.add_component( "w%s_v%s" % (u_idx, v_idx), _w)

        K = max(b.DISCRETIZATION)

        _dw = Var(bounds=( min(0, _lb*2**-K, _ub*2**-K), 
                           max(0, _lb*2**-K, _ub*2**-K) ))
        b.add_component( "dw%s_v%s" % (u_idx, v_idx), _dw)

        _c = Constraint(expr= _w == _v_lb*u + (_v_ub-_v_lb) * (
                sum(2**-k * _u[k] for k in b.DISCRETIZATION) + _dw ) )
        b.add_component( "c_bilinear_u%s_v%s" % (u_idx, v_idx), _c )

        _c = ConstraintList(noruleinit=True)
        b.add_component( "c_mccormick_u%s_v%s" % (u_idx, v_idx), _c )
        # u_lb * dv <= dw <= u_ub * dv
        _c.add(expr= _lb*_dv <= _dw )
        _c.add(expr= _dw <= _ub*_dv )
        # (u-u_ub)*2^-K + u_ub*dv <= dw <= (u-u_lb)*2^-K + u_lb*dv
        _c.add(expr= (u - _ub)*2**-K + _ub*_dv <= _dw )
        _c.add(expr= _dw <= (u - _lb)*2**-K + _lb*_dv )

        return _w

    def _collect_bilinear(self, expr, bilin, quad):
        if not expr.is_expression():
            return
        if type(expr) is _ProductExpression:
            if len(expr._numerator) != 2:
                for e in expr._numerator:
                    self._collect_bilinear(e, bilin, quad)
                # No need to check denominator, as this is poly_degree==2
                return
            if not isinstance(expr._numerator[0], _VarData) or \
                    not isinstance(expr._numerator[1], _VarData):
                raise RuntimeError("Cannot yet handle complex subexpressions")
            if expr._numerator[0] is expr._numerator[1]:
                quad.append( (expr, expr._numerator[0]) )
            else:
                bilin.append( (expr, expr._numerator[0], expr._numerator[1]) )
            return
        if type(expr) is _PowExpression and value(expr._args[1]) == 2:
            # Note: directly testing the value of the exponent above is
            # safe: we have already verified that this expression is
            # polynominal, so the exponent must be constant.
            tmp = _ProductExpression()
            tmp._numerator = [ expr._args[0], expr._args[0] ]
            tmp._denominator = []
            expr._args = (tmp, as_numeric(1))
            #quad.append( (tmp, tmp._args[0]) )
            self._collect_bilinear(tmp, bilin, quad)
            return 
        # All other expression types
        for e in expr._args:
            self._collect_bilinear(e, bilin, quad)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# AMPL Problem Writer Plugin
#

__all__ = ['ProblemWriter_nl']

from coopr.opt import ProblemFormat, WriterFactory
from coopr.opt.base import *
from coopr.pyomo.base.objective import minimize, maximize
from coopr.pyomo.base import *
from coopr.pyomo.base import expr, external, SymbolMap, Block
from coopr.pyomo.base import intrinsic_functions
from coopr.pyomo.base.var import _VarData, Var
from coopr.pyomo.base import _ExpressionData, Expression
from coopr.pyomo.base.numvalue import NumericConstant, native_numeric_types
from coopr.pyomo.base.param import _ParamData
from coopr.pyomo.base import var
from coopr.pyomo.base import param
from coopr.pyomo.base import numvalue
from coopr.pyomo.base.suffix import active_export_suffix_generator
from coopr.pyomo.expr.ampl_repn import generate_ampl_repn
from coopr.core.plugin import alias
from pyutilib.misc import PauseGC

import itertools
import logging
import operator
import os
import time
import sys

from six import itervalues, iteritems, iterkeys, StringIO
from six.moves import xrange, zip

logger = logging.getLogger('coopr.pyomo')

intrinsic_function_operators = {\
'log':    'o43',\
'log10':  'o42',\
'sin':    'o41',\
'cos':    'o46',
'tan':    'o38',\
'sinh':   'o40',\
'cosh':   'o45',\
'tanh':   'o37',\
'asin':   'o51',\
'acos':   'o53',\
'atan':   'o49',\
'exp':    'o44',\
'sqrt':   'o39',\
'asinh':  'o50',\
'acosh':  'o52',\
'atanh':  'o47',\
'pow':    'o5',\
'abs':    'o15'}


class StopWatch(object):

    def __init__(self):
        self.start = time.time()
            
    def report(self, msg):
        print(msg+" (seconds): "+str(time.time()-self.start))

    def reset(self):
        self.start = time.time()


class _Counter(object):

    def __init__(self, start):
        self._id = start

    def __call__(self, obj):
        tmp = self._id
        self._id += 1
        return tmp


class ModelSOS(object):

    class AmplSuffix(object):

        def __init__(self,name):
            self.name = name
            self.ids = []
            self.vals = []

        def add(self,idx,val):
            if idx in self.ids:
                raise RuntimeError("The NL file format does not support multiple nonzero "\
                                    "values for a single component and suffix. \n"\
                                    "Suffix Name:  %s\n"\
                                    "Component ID: %s\n" % (self.name, idx))
            else:
                self.ids.append(idx)
                self.vals.append(val)
                    
        def genfilelines(self):
            base_line = "{0} {1}\n"
            return [base_line.format(idx, val) for idx, val in zip(self.ids,self.vals) if val != 0]

        def is_empty(self):
            return not bool(len(self.ids))

    def __init__(self,ampl_var_id, varID_map):

        self.ampl_var_id = ampl_var_id
        self.sosno = self.AmplSuffix('sosno')
        self.ref = self.AmplSuffix('ref')
        self.block_cntr = 0
        self.varID_map = varID_map

    def count_constraint(self,soscondata):
        
        ampl_var_id = self.ampl_var_id
        varID_map = self.varID_map

        sos_items = soscondata.get_items()
        level = soscondata.get_level()

        if len(sos_items) == 0:
            return

        # Identifies the the current set from others in the NL file
        self.block_cntr += 1

        # If SOS1, the identifier must be positive
        # if SOS2, the identifier must be negative
        sign_tag = None
        if level == 1:
            sign_tag = 1
        elif level == 2:
            sign_tag = -1
        else:
            raise ValueError("SOSContraint '%s' has sos type='%s', "
                             "which is not supported by the NL file interface" \
                                 % (soscondata.cname(True), level))

        for vardata, weight in sos_items:
            if vardata.fixed:
                raise RuntimeError("SOSConstraint '%s' includes a fixed Variable '%s'. "
                                   "This is currently not supported. Deactivate this constraint "
                                   "in order to proceed" % (soscondata.cname(True), vardata.cname(True)))
            ID = ampl_var_id[varID_map[id(vardata)]]
            self.sosno.add(ID,self.block_cntr*sign_tag)
            self.ref.add(ID,weight)


class RepnWrapper(object):

    __slots__ = ('repn','_linear_vars','_nonlinear_vars')

    def __init__(self,repn,linear,nonlinear):
        self.repn = repn
        self._linear_vars = linear
        self._nonlinear_vars = nonlinear


class ProblemWriter_nl(AbstractProblemWriter):

    alias(str(ProblemFormat.nl))

    def __init__(self):
        self._ampl_var_id = {}
        self._ampl_con_id = {}
        self._ampl_obj_id = {}
        self._OUTPUT = None
        self._varID_map = None
        AbstractProblemWriter.__init__(self, ProblemFormat.nl)

    def __call__(self, model, filename, solver_capability, io_options, **kwds):
        # Pause the GC for the duration of this method
        suspend_gc = PauseGC()

        # NOTE: io_options is a simple dictionary of keyword-value pairs
        #       specific to this writer. that said, we are not good
        #       about enforcing consistency between the io_options and
        #       kwds - arguably, we should go with the former or the
        #       latter, but not support both. then again, the kwds are
        #       not used anywhere within this function.

        # Print timing after writing each section of the NL file
        show_section_timing = io_options.pop("show_section_timing", False)

        # Skip writing constraints whose body section is fixed (i.e., no variables)
        skip_trivial_constraints = io_options.pop("skip_trivial_constraints", False)

        # Write NL files deterministically. The main source of non-determinism comes
        # from indexed components. This flag injects a few sort_by_keys=True statements
        # into the active_subcomponent_data_generators to ensure identical NL files are written
        # across different python builds. Set to false for potential speed improvements.
        deterministic = io_options.pop("deterministic", True)

        # Write the corresponding .row and .col files for the NL files identifying variable
        # and constraint indices in the NLP matrix.
        symbolic_solver_labels = io_options.pop("symbolic_solver_labels", False)

        # If False, we raise an error if fixed variables are encountered
        # in the list of active variables (i.e., an expression containing some fixed variable
        # was not preprocessed after the variable was fixed). If True, we allow this
        # case and modify the variable bounds section to fix the variable.
        output_fixed_variable_bounds = io_options.pop("output_fixed_variable_bounds", False)

        if io_options:
            logger.warn(
                "ProblemWriter_nl passed unrecognized io_options:\n\t" +
                "\n\t".join("%s = %s" % (k,v) for k,v in iteritems(io_options)))

        if filename is None:
            filename = model.cname() + ".nl"
        with open(filename,"w") as f:
            self._OUTPUT = f
            symbol_map = self._print_model_NL(model,
                                              solver_capability,
                                              symbolic_solver_labels,
                                              show_section_timing=show_section_timing,
                                              skip_trivial_constraints=skip_trivial_constraints,
                                              deterministic=deterministic,
                                              output_fixed_variable_bounds=output_fixed_variable_bounds)
        self._OUTPUT = None
        self._varID_map = None
        return filename, symbol_map

    def _get_bound(self, exp):
        if exp.is_fixed():
            return exp()
        else:
            raise ValueError("ERROR: non-fixed bound: " + str(exp))

    def _print_nonlinear_terms_NL(self, exp):
        OUTPUT = self._OUTPUT
        exp_type = type(exp)
        # JDS: check list first so that after this, we know that exp
        # must be some form of NumericValue
        if exp_type is list:
            # this is an implied summation of expressions (we did not create a new sum expression for efficiency)
            # this should be a list of tuples where [0] is the coeff and [1] is the expr to write
            n = len(exp)
            if n > 2:
                # sum
                OUTPUT.write( "o54\n%d\n"%(n) )
                for i in xrange(0,n):
                    assert(exp[i].__class__ is tuple)
                    coef = exp[i][0]
                    child_exp = exp[i][1]
                    if coef != 1.0:
                        # *
                        OUTPUT.write( "o2\nn{0!r}\n".format(coef) )
                    self._print_nonlinear_terms_NL(child_exp)
            else:
                for i in xrange(0,n):
                    assert(exp[i].__class__ is tuple)
                    coef = exp[i][0]
                    child_exp = exp[i][1]
                    if i != n-1:
                        # need the + op if it is not the last entry in the list
                        # +
                        OUTPUT.write( "o0\n" )
                    if coef != 1.0:
                        # *
                        OUTPUT.write( "o2\nn{0!r}\n".format(coef) )
                    self._print_nonlinear_terms_NL(child_exp)
        elif exp.is_expression():
            if exp_type is expr._SumExpression:
                n = len(exp._args)
                if n > 2:
                    # sum
                    OUTPUT.write( "o54\n%d\n"%(n) )
                    if exp._const != 0.0:
                        # +
                        OUTPUT.write( "o0\nn{0!r}\n".format(exp._const) )
                    for i in xrange(0,n):
                        if exp._coef[i] != 1:
                            # *
                            OUTPUT.write( "o2\nn{0!r}\n".format(exp._coef[i]) )
                        self._print_nonlinear_terms_NL(exp._args[i])
                else:
                    if exp._const != 0.0:
                        # +
                        OUTPUT.write( "o0\nn{0!r}\n".format(exp._const) )
                    for i in xrange(0,n-1):
                        # +
                        OUTPUT.write( "o0\n" )
                        if exp._coef[i] != 1.0:
                            # *
                            OUTPUT.write( "o2\nn{0!r}\n".format(exp._coef[i]) )
                        self._print_nonlinear_terms_NL(exp._args[i])
                    if exp._coef[n-1] != 1.0:
                        # *
                        OUTPUT.write( "o2\nn{0!r}\n".format(exp._coef[n-1]) )
                    self._print_nonlinear_terms_NL(exp._args[n-1])
    
            elif exp_type is expr._ProductExpression:
                denom_exists = False
                if len(exp._denominator) == 0:
                    pass
                else:
                    # /
                    OUTPUT.write( "o3\n" )
                    denom_exists = True
                if exp._coef != 1.0:
                    # *
                    OUTPUT.write( "o2\nn{0!r}\n".format(exp._coef) )
                if len(exp._numerator) == 0:
                    OUTPUT.write( "n1\n" )
                # print out the numerator
                child_counter = 0
                max_count = len(exp._numerator)-1
                for child_exp in exp._numerator:
                    if child_counter < max_count:
                        # *
                        OUTPUT.write( "o2\n" )
                    self._print_nonlinear_terms_NL(child_exp)
                    child_counter += 1
                if denom_exists:
                    # print out the denominator
                    child_counter = 0
                    max_count = len(exp._denominator)-1
                    for child_exp in exp._denominator:
                        if child_counter < max_count:
                            # *
                            OUTPUT.write( "o2\n" )
                        self._print_nonlinear_terms_NL(child_exp)
                        child_counter += 1
    
            elif exp_type is external._ExternalFunctionExpression:
                OUTPUT.write("f%d %d\n" % (
                        self.external_byFcn[exp._fcn._function][1], 
                        len(exp._args) ) )
                for arg in exp._args:
                    if isinstance(arg, basestring):
                        OUTPUT.write("h%d:%s\n" % (len(arg), arg))
                    else:
                        self._print_nonlinear_terms_NL(arg)
            elif isinstance(exp, expr._IntrinsicFunctionExpression):
                intr_expr = intrinsic_function_operators.get(exp.cname(),None)
                if intr_expr is not None:
                    OUTPUT.write( intr_expr+"\n" )
                else:
                    logger.error("Unsupported intrinsic function ({0})", exp.cname(True))
                    raise TypeError("ASL writer does not support '{0}' expressions"
                                   .format(exp.cname(True)))
                    
                for child_exp in exp._args:
                    self._print_nonlinear_terms_NL(child_exp)
            elif exp_type is expr.Expr_if:
                OUTPUT.write("o35\n")
                self._print_nonlinear_terms_NL(exp._if)
                self._print_nonlinear_terms_NL(exp._then)
                self._print_nonlinear_terms_NL(exp._else)
            elif exp_type is expr._InequalityExpression:
                len_args = len(exp._args)
                assert len_args in [2,3]
                left = exp._args[0]
                middle = exp._args[1]
                right = None
                if len_args == 3:
                    right = exp._args[2]
                    OUTPUT.write("o21\n") # and
                if exp._strict[0]:
                    OUTPUT.write("o22\n")
                else:
                    OUTPUT.write("o23\n")
                self._print_nonlinear_terms_NL(left)
                self._print_nonlinear_terms_NL(middle)
                if not right is None:
                    if exp._strict[1]:
                        OUTPUT.write("o22\n")
                    else:
                        OUTPUT.write("o23\n")
                    self._print_nonlinear_terms_NL(middle)
                    self._print_nonlinear_terms_NL(right)
            elif exp_type is expr._EqualityExpression:
                OUTPUT.write("o24\n")
                self._print_nonlinear_terms_NL(exp._args[0])
                self._print_nonlinear_terms_NL(exp._args[1])
            elif exp_type is _ExpressionData or exp.type() is Expression:
                self._print_nonlinear_terms_NL(exp.value)
            else:
                raise ValueError("Unsupported expression type (%s) in _print_nonlinear_terms_NL" % exp_type)

        elif isinstance(exp,var._VarData) and not exp.is_fixed():
            OUTPUT.write( "v{0}\n".format(self.ampl_var_id[self._varID_map[id(exp)]]) )
        elif isinstance(exp,param._ParamData):
            OUTPUT.write( "n{0!r}\n".format(value(exp)) )
        elif isinstance(exp,NumericConstant) or exp.is_fixed():
            OUTPUT.write( "n{0!r}\n".format(value(exp)) )
        else:
            raise ValueError("Unsupported expression type (%s) in _print_nonlinear_terms_NL" % exp_type)
        
    
    def _print_model_NL(self, model, 
                        solver_capability,
                        symbolic_solver_labels,
                        show_section_timing=False,
                        skip_trivial_constraints=False,
                        deterministic=True,
                        output_fixed_variable_bounds=False):

        OUTPUT = self._OUTPUT
        assert OUTPUT is not None

        # maps NL variables to the "real" variable names in the problem.
        # it's really NL variable ordering, as there are no variable names
        # in the NL format. however, we by convention make them go from
        # x0 upward.

        overall_timer = StopWatch()
        subsection_timer = StopWatch()

        # create the symbol_map
        symbol_map = SymbolMap(model)

        # Speeds up calling cname on every component when
        # writing .row and .col files (when symbolic_solver_labels is True)
        name_labeler = NameLabeler()

        #
        # Collect statistics
        #
        self_ampl_var_id = self.ampl_var_id = {}
        self_ampl_con_id = self.ampl_con_id = {}
        self_ampl_obj_id = self.ampl_obj_id = {}

        Vars_dict = dict() # will be the entire list of all vars in the model (not necessarily used) 
        Objectives_dict = dict() # will be the entire list of all objective used in the problem (size=1)
        Constraints_dict = dict() # will be the entire list of all constraints used in the problem

        # linear variables
        LinearVars = set()
        LinearVarsInt = set()
        LinearVarsBool = set()

        # Tabulate the External Function definitions
        self.external_byFcn = {}
        external_Libs = set()
        for block in model.all_blocks():
            for fcn in active_subcomponents_generator(block,ExternalFunction):
                if fcn._function in self.external_byFcn and \
                        self.external_byFcn[fcn._function][0]._library != fcn._library:
                    raise RuntimeError(
                        "The same external function name (%s) is associated "
                        "with two different libraries (%s through %s, and %s "
                        "through %s).  The ASL solver will fail to link "
                        "correctly." %
                        ( fcn._function, 
                          self.external_byFcn[fcn._function]._library,
                          self.external_byFcn[fcn._function]._library.cname(True),
                          fcn._library,
                          fcn.cname(True) ) )
                self.external_byFcn[fcn._function] = (fcn, len(self.external_byFcn))
                external_Libs.add(fcn._library)
        if external_Libs:
            os.environ["COOPR_AMPLFUNC"] = "\n".join(sorted(external_Libs))
        elif "COOPR_AMPLFUNC" in os.environ:
            del os.environ["COOPR_AMPLFUNC"]

        subsection_timer.reset()

        # converting complementarity conditions to standard form
        from coopr.mpec import Complementarity
        for block in model.all_blocks():
            for active in active_subcomponents_data_generator(block,Complementarity):
                active.to_standard_form()
                block.reclassify_subcomponent_type(active, Block)

        # Cache the list of model blocks so we don't have to call
        # model.all_blocks() many many times
        all_blocks_list = list(model.all_blocks(sort_by_keys=deterministic))

        # create a deterministic var labeling
        cntr = 0
        for block in all_blocks_list:
            vars_counter = tuple(enumerate(active_subcomponents_data_generator(block, Var, sort_by_keys=deterministic), cntr))
            cntr += len(vars_counter)+1
            Vars_dict.update(vars_counter)
        self._varID_map = dict((id(val),key) for key,val in iteritems(Vars_dict))
        self_varID_map = self._varID_map
        # Use to label the rest of the components (which we will not encounter twice)
        trivial_labeler = _Counter(cntr)

        #
        # Count number of objectives and build the ampl_repns
        #
        n_objs = 0
        n_nonlinear_objs = 0
        ObjVars = set()
        ObjNonlinearVars = set()
        ObjNonlinearVarsInt = set()
        for block in all_blocks_list:
            gen_obj_ampl_repn = getattr(model,"gen_obj_ampl_repn",True)

            # Create a suffix component for the ampl_repn
            block_ampl_repn_suffix = block.subcomponent("_ampl_repn")
            if (block_ampl_repn_suffix is None) or (not block_ampl_repn_suffix.type() is Suffix) or (not block_ampl_repn_suffix.active is True):
                block.del_component("_ampl_repn")
                block_ampl_repn_suffix = block._ampl_repn = Suffix(direction=Suffix.LOCAL,datatype=None)
                
            for active_objective in active_subcomponents_data_generator(block,Objective,sort_by_keys=deterministic):
        
                if gen_obj_ampl_repn is True:
                    ampl_repn = generate_ampl_repn(active_objective.expr)
                    block_ampl_repn_suffix.setValue(active_objective, ampl_repn)
                else:
                    ampl_repn = block_ampl_repn_suffix.getValue(active_objective)
                    
                wrapped_ampl_repn = RepnWrapper(ampl_repn,
                                                list(self_varID_map[id(var)] for var in ampl_repn._linear_vars),
                                                list(self_varID_map[id(var)] for var in ampl_repn._nonlinear_vars))
            
                LinearVars.update(wrapped_ampl_repn._linear_vars)
                ObjNonlinearVars.update(wrapped_ampl_repn._nonlinear_vars)
            
                ObjVars.update(wrapped_ampl_repn._linear_vars)
                ObjVars.update(wrapped_ampl_repn._nonlinear_vars)

                obj_ID = trivial_labeler(active_objective)
                Objectives_dict[obj_ID] = (active_objective, wrapped_ampl_repn)
                self_ampl_obj_id[obj_ID] = n_objs
                symbol_map.updateSymbols([(active_objective, "o%d"%n_objs)])

                n_objs += 1
                if ampl_repn.is_nonlinear():
                    n_nonlinear_objs += 1

        # I don't think this is necessarily true for the entire code base, 
        # but seeing has how its never been tested we should go ahead and
        # raise an exception
        if n_objs > 1:
            raise ValueError("The NL writer has detected multiple active objective functions on model %s, "
                             "but currently only handles a single objective." % (model.cname(True)))
        elif n_objs == 1:
            symbol_map.alias(symbol_map.bySymbol["o0"](),"__default_objective__")

        if show_section_timing:
            subsection_timer.report("Generate objective representation")
            subsection_timer.reset()

        #
        # Count number of constraints and build the ampl_repns
        #
        n_ranges = 0
        n_single_sided_ineq = 0
        n_equals = 0
        n_nonlinear_constraints = 0
        ConNonlinearVars = set()
        ConNonlinearVarsInt = set()
        nnz_grad_constraints = 0
        constraint_bounds_dict = {}
        nonlin_con_order_list = []
        lin_con_order_list = []        

        for block in all_blocks_list:
            all_repns = list()
            
            gen_con_ampl_repn = getattr(block,"gen_con_ampl_repn",True)

            # Create a suffix component for the ampl_repn
            block_ampl_repn_suffix = block.subcomponent("_ampl_repn")
            if (block_ampl_repn_suffix is None) or (not block_ampl_repn_suffix.type() is Suffix) or (not block_ampl_repn_suffix.active is True):
                block.del_component("_ampl_repn")
                block_ampl_repn_suffix = block._ampl_repn = Suffix(direction=Suffix.LOCAL,datatype=None)

            # Initializing the constraint dictionary
            for constraint_data in active_subcomponents_data_generator(block,Constraint,sort_by_keys=deterministic):
                if gen_con_ampl_repn is True:
                    ampl_repn = generate_ampl_repn(constraint_data.body)
                    all_repns.append((constraint_data,ampl_repn))
                else:
                    ampl_repn = block_ampl_repn_suffix.getValue(constraint_data)
                
                ### GAH: Even if this is fixed, it is still useful to write out these types of constraints
                ###      (trivial) as a feasibility check for fixed variables, in which case the solver
                ###      will pick up on the model infeasibility.
                if skip_trivial_constraints and ampl_repn.is_fixed():
                    continue

                con_ID = trivial_labeler(constraint_data)
                wrapped_ampl_repn = RepnWrapper(ampl_repn,
                                                list(self_varID_map[id(var)] for var in ampl_repn._linear_vars),
                                                list(self_varID_map[id(var)] for var in ampl_repn._nonlinear_vars))
                    
                if ampl_repn.is_nonlinear():
                    nonlin_con_order_list.append(con_ID)
                    n_nonlinear_constraints += 1
                else:
                    lin_con_order_list.append(con_ID)

                Constraints_dict[con_ID] = (constraint_data, wrapped_ampl_repn)
                
                LinearVars.update(wrapped_ampl_repn._linear_vars)
                ConNonlinearVars.update(wrapped_ampl_repn._nonlinear_vars)
                    
                nnz_grad_constraints += len(set(wrapped_ampl_repn._linear_vars).union(wrapped_ampl_repn._nonlinear_vars))
    
                L = None
                U = None
                if constraint_data.lower is not None: L = self._get_bound(constraint_data.lower)
                if constraint_data.upper is not None: U = self._get_bound(constraint_data.upper)

                offset = ampl_repn._constant
                #if constraint_data._equality is True:
                #    assert L == U
                try:
                    _type = constraint_data._complementarity
                    _vid = self_varID_map[id(block.v)]
                    constraint_bounds_dict[con_ID] = "5 {0} {1}\n".format(_type, _vid)
                except AttributeError:
                    if L == U:
                        constraint_bounds_dict[con_ID] = "4 {0!r}\n".format(L-offset)
                        n_equals += 1
                    elif L is None:
                        constraint_bounds_dict[con_ID] = "1 {0!r}\n".format(U-offset)
                        n_single_sided_ineq += 1
                    elif U is None:
                        constraint_bounds_dict[con_ID] = "2 {0!r}\n".format(L-offset)
                        n_single_sided_ineq += 1
                    elif (L > U):
                        msg = 'Constraint {0}: lower bound greater than upper' \
                            ' bound ({1} > {2})'
                        raise ValueError(msg.format(con_ID, str(L), str(U)))
                    else:
                        constraint_bounds_dict[con_ID] = "0 {0!r} {1!r}\n".format(L-offset, U-offset)
                        # double sided inequality
                        # both are not none and they are valid
                        n_ranges += 1
            block_ampl_repn_suffix.updateValues(all_repns)

        sos1 = solver_capability("sos1")
        sos2 = solver_capability("sos2")
        for block in all_blocks_list:
            for soscondata in active_subcomponents_data_generator(block, SOSConstraint, sort_by_keys=deterministic):
                level = soscondata.get_level()
                if (level == 1 and not sos1) or (level == 2 and not sos2):
                    raise Exception(
                        "Solver does not support SOS level %s constraints"
                        % (level,) )
                LinearVars.update(self_varID_map[id(vardata)] for vardata in soscondata.get_members())

        # create the ampl constraint ids
        self_ampl_con_id.update((con_ID,row_id) for row_id,con_ID in \
                                enumerate(itertools.chain(nonlin_con_order_list,lin_con_order_list)))
        # populate the symbol_map
        symbol_map.updateSymbols( [(Constraints_dict[con_ID][0],"c%d"%row_id) for row_id,con_ID in \
                                   enumerate(itertools.chain(nonlin_con_order_list,lin_con_order_list))] )

        if show_section_timing:
            subsection_timer.report("Generate constraint representations")
            subsection_timer.reset()

        LinearVars = LinearVars.difference(ObjNonlinearVars)
        LinearVars = LinearVars.difference(ConNonlinearVars)

        ### There used to be an if statement here for the following code block 
        ### checking model.statistics.num_binary_vars was greater than zero.
        ### To this day, I don't know how it worked.
        ### TODO: Figure out why
        ###############
        for var_ID in LinearVars:
            var = Vars_dict[var_ID]
            if var.is_integer():
                LinearVarsInt.add(var_ID)
            elif var.is_binary():
                L = var.lb
                U = var.ub
                if L is None or U is None:
                    raise ValueError("Variable " + str(var.cname(True)) +\
                                     "is binary, but does not have lb and ub set")
                LinearVarsBool.add(var_ID)
            elif not var.is_continuous():
                raise TypeError("Invalid domain type for variable with name '%s'. "
                                "Variable is not continuous, integer, or binary.")
        LinearVars.difference_update(LinearVarsInt)
        LinearVars.difference_update(LinearVarsBool)

        for var_ID in ObjNonlinearVars:
            var = Vars_dict[var_ID]
            if var.is_integer() or var.is_binary():
                ObjNonlinearVarsInt.add(var_ID)
            elif not var.is_continuous():
                raise TypeError("Invalid domain type for variable with name '%s'. "
                                "Variable is not continuous, integer, or binary.")
        ObjNonlinearVars.difference_update(ObjNonlinearVarsInt)
        for var_ID in ConNonlinearVars:
            var = Vars_dict[var_ID]
            if var.is_integer() or var.is_binary():
                ConNonlinearVarsInt.add(var_ID)
            elif not var.is_continuous():
                raise TypeError("Invalid domain type for variable with name '%s'. "
                                "Variable is not continuous, integer, or binary.")
        ConNonlinearVars.difference_update(ConNonlinearVarsInt)
        ##################

        Nonlinear_Vars_in_Objs_and_Constraints = ObjNonlinearVars.intersection(ConNonlinearVars)
        Discrete_Nonlinear_Vars_in_Objs_and_Constraints = ObjNonlinearVarsInt.intersection(ConNonlinearVarsInt)
        ObjNonlinearVars = ObjNonlinearVars.difference(Nonlinear_Vars_in_Objs_and_Constraints)
        ConNonlinearVars = ConNonlinearVars.difference(Nonlinear_Vars_in_Objs_and_Constraints)
        ObjNonlinearVarsInt = ObjNonlinearVarsInt.difference(Discrete_Nonlinear_Vars_in_Objs_and_Constraints)
        ConNonlinearVarsInt = ConNonlinearVarsInt.difference(Discrete_Nonlinear_Vars_in_Objs_and_Constraints)
        
        # put the ampl variable id into the variable
        full_var_list = []
        full_var_list.extend(sorted(Nonlinear_Vars_in_Objs_and_Constraints))
        full_var_list.extend(sorted(Discrete_Nonlinear_Vars_in_Objs_and_Constraints))
        idx_nl_both = len(full_var_list)
        #
        full_var_list.extend(sorted(ConNonlinearVars))
        full_var_list.extend(sorted(ConNonlinearVarsInt))
        idx_nl_con = len(full_var_list)
        #
        full_var_list.extend(sorted(ObjNonlinearVars))
        full_var_list.extend(sorted(ObjNonlinearVarsInt))
        idx_nl_obj = len(full_var_list)
        #
        full_var_list.extend(sorted(LinearVars))
        full_var_list.extend(sorted(LinearVarsBool))
        full_var_list.extend(sorted(LinearVarsInt))

        if (idx_nl_obj == idx_nl_con):
            idx_nl_obj = idx_nl_both

        # create the ampl variable column ids
        self_ampl_var_id.update( (var_ID,column_id) for column_id,var_ID in enumerate(full_var_list))
        # populate the symbol_map
        symbol_map.updateSymbols( [(Vars_dict[var_ID],"v%d"%column_id) for column_id,var_ID in enumerate(full_var_list)] )

        if show_section_timing:
            subsection_timer.report("Partition variable types")
            subsection_timer.reset()

        ##################################################
        ############ # LQM additions start here ##########
        ##################################################

        ###                                                                                                    
        # Generate file containing matrices for continuity variables between blocks.                            
        # This checks for a LOCAL suffix 'lqm' on variables.
        ###
        lqm_suffix = model.subcomponent("lqm")
        if (lqm_suffix is None) or (not (lqm_suffix.type() is Suffix)) or \
           (not (lqm_suffix.active is True)) or (not (lqm_suffix.getDirection() is Suffix.LOCAL)):
            lqm_suffix = None
        if lqm_suffix is not None:
            lqm_var_column_ids = []
            for var_ID in full_var_list:
                lqm = lqm_suffix.getValue(Vars_dict[var_ID])
                if lqm != -1:
                    # store the column index (and translate to one-based indexing
                    lqm_var_column_ids.append(self_ampl_var_id[var_ID]+1)

            num_lqm_vars = len(lqm_var_column_ids)
            if num_lqm_vars > 0:
                lqm_output_name = OUTPUT.name.split('.')[0]+".lqm"
                LQM_OUTPUT = open(lqm_output_name,'w')
                LQM_OUTPUT.write( "Matrix Li\n" )
                LQM_OUTPUT.write( "%d\n"%(len(full_var_list)) )
                LQM_OUTPUT.write( "%d\n"%(num_lqm_vars) )
                LQM_OUTPUT.write( "%d\n"%(num_lqm_vars) )
                # The matrix uses one based indexing
                for row_id, col_id in enumerate(lqm_var_column_ids,1):
                    LQM_OUTPUT.write( "%d %d 1\n" % (row_id, col_id) )
                LQM_OUTPUT.write( "Matrix Qi\n" )
                LQM_OUTPUT.write("%d\n" % (num_lqm_vars) )
                LQM_OUTPUT.write("%d\n" % (num_lqm_vars) )
                LQM_OUTPUT.write("%d\n" % (num_lqm_vars) )
                # one based indexing
                for counter in xrange(1,num_lqm_vars+1):
                    LQM_OUTPUT.write("%d %d -1\n" % (counter, counter) )
                LQM_OUTPUT.close()

        ##################################################
        ############ # LQM additions end here ############
        ##################################################

#        end_time = time.clock()
#        print (end_time - start_time)

        colfilename = None
        if OUTPUT.name.endswith('.nl'):
            colfilename = OUTPUT.name.replace('.nl','.col')
        else:
            colfilename = OUTPUT.name+'.col'
        if symbolic_solver_labels is True:
            colf = open(colfilename,'w')
            for var_ID in full_var_list:
                colf.write(name_labeler(Vars_dict[var_ID])+'\n')
            colf.close()
        
        if show_section_timing:
            subsection_timer.report("Write .col file")
            subsection_timer.reset()

        #
        # Print Header
        #
        # LINE 1
        #
        OUTPUT.write("g3 1 1 0\t# problem {0}\n".format(model.cname()) )
        #
        # LINE 2
        #
        OUTPUT.write( " {0} {1} {2} {3} {4} \t# vars, constraints, objectives, general inequalities, equalities\n" \
                          .format(len(full_var_list), n_single_sided_ineq+n_ranges+n_equals, n_objs, n_ranges, n_equals) )
        #
        # LINE 3
        #
        OUTPUT.write( " {0} {1} \t# nonlinear constraints, objectives\n".format(n_nonlinear_constraints, n_nonlinear_objs) )
        #
        # LINE 4
        #
        OUTPUT.write( " 0 0\t# network constraints: nonlinear, linear\n" )
        #
        # LINE 5
        #
        OUTPUT.write( " {0} {1} {2} \t# nonlinear vars in constraints, objectives, both\n" \
                          .format(idx_nl_con, idx_nl_obj, idx_nl_both) )

        #
        # LINE 6
        #
        OUTPUT.write( " 0 {0} 0 1\t# linear network variables; functions; arith, flags\n".format(len(self.external_byFcn)) )
        #
        # LINE 7
        #
        n_int_nonlinear_b = len(Discrete_Nonlinear_Vars_in_Objs_and_Constraints)
        n_int_nonlinear_c = len(ConNonlinearVarsInt)
        n_int_nonlinear_o = len(ObjNonlinearVarsInt) 
        OUTPUT.write( " {0} {1} {2} {3} {4} \t# discrete variables: binary, integer, nonlinear (b,c,o)\n" \
           .format(len(LinearVarsBool), len(LinearVarsInt), n_int_nonlinear_b, n_int_nonlinear_c, n_int_nonlinear_o) )
        #
        # LINE 8
        #
        # objective info computed above
        OUTPUT.write( " {0} {1} \t# nonzeros in Jacobian, obj. gradient\n".format(nnz_grad_constraints, len(ObjVars)) )
        #
        # LINE 9
        #
        OUTPUT.write( " 0 0\t# max name lengths: constraints, variables\n" )
        #
        # LINE 10
        #
        OUTPUT.write( " 0 0 0 0 0\t# common exprs: b,c,o,c1,o1\n" )

#        end_time = time.clock()
#        print (end_time - start_time)

#        print "Printing constraints:",
#        start_time = time.clock()


        #
        # "F" lines
        #
        for fcn, fid in sorted(itervalues(self.external_byFcn), key=operator.itemgetter(1)):
            OUTPUT.write( "F%d 1 -1 %s\n" % ( fid, fcn._function ))

        #
        # "S" lines
        #

        # Tranlate the SOSConstraint component into ampl suffixes
        sos1 = solver_capability("sos1")
        sos2 = solver_capability("sos2")
        modelSOS = ModelSOS(self_ampl_var_id, self_varID_map)
        for block in all_blocks_list:
            for soscondata in active_subcomponents_data_generator(block, SOSConstraint, sort_by_keys=deterministic):
                level = soscondata.get_level()
                if (level == 1 and not sos1) or (level == 2 and not sos2):
                    raise ValueError("Solver does not support SOS level %s constraints"
                                     % (level,) )
                modelSOS.count_constraint(soscondata)

        var_sosno_suffix = modelSOS.sosno
        var_ref_suffix = modelSOS.ref
        sosconstraint_sosno_vals = set(var_sosno_suffix.vals)

        # Translate the rest of the Pyomo Suffix components
        suffix_header_line = "S{0} {1} {2}\n"
        suffix_line = "{0} {1!r}\n"
        var_tag = 0
        con_tag = 1
        obj_tag = 2
        prob_tag = 3
        suffix_dict = {}
        for block in all_blocks_list:
            for name, suf in active_export_suffix_generator(block):
                suffix_dict.setdefault(name,[]).append(suf)
        if not ('sosno' in suffix_dict):
            # We still need to write out the SOSConstraint suffixes
            # even though these may have not been "declared" on the model
            s_lines = var_sosno_suffix.genfilelines()
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write(suffix_header_line.format(var_tag,len_s_lines,'sosno'))
                OUTPUT.writelines(s_lines)
        else:
            # I am choosing not to allow a user to mix the use of the Pyomo
            # SOSConstraint component and manual sosno declarations within
            # a single model. I initially tried to allow this but the
            # var section of the code below blows up for two reason. (1)
            # we have to make sure that the sosno suffix is not defined
            # twice for the same variable (2) We have to make sure that
            # the automatically chosen sosno used by the SOSConstraint
            # translation does not already match one a user has manually
            # implemented (this would modify the members in an sos set).
            # Since this suffix is exclusively used for defining sos sets,
            # there is no reason a user can not just stick to one method.
            if not var_sosno_suffix.is_empty():
                raise RuntimeError("The Pyomo NL file writer does not allow both manually "\
                                    "declared 'sosno' suffixes as well as SOSConstraint "\
                                    "components to exist on a single model. To avoid this "\
                                    "error please use only one of these methods to define "\
                                    "special ordered sets.")
        if not ('ref' in suffix_dict):
            # We still need to write out the SOSConstraint suffixes
            # even though these may have not been "declared" on the model
            s_lines = var_ref_suffix.genfilelines()
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write(suffix_header_line.format(var_tag,len_s_lines,'ref'))
                OUTPUT.writelines(s_lines)
        else:
            # see reason (1) in the paragraph above for why we raise this
            # exception (replacing sosno with ref).
            if not var_ref_suffix.is_empty():
                raise RuntimeError("The Pyomo NL file writer does not allow both manually "\
                                    "declared 'ref' suffixes as well as SOSConstraint "\
                                    "components to exist on a single model. To avoid this "\
                                    "error please use only one of these methods to define "\
                                    "special ordered sets.")
        for suffix_name, suffixes in iteritems(suffix_dict): 
            datatypes = set()
            for suffix in suffixes:
                datatype = suffix.getDatatype()
                if datatype not in (Suffix.FLOAT,Suffix.INT):
                    raise ValueError("The Pyomo NL file writer requires that all active export Suffix components "\
                                     "declare a numeric datatype. Suffix component: %s with " % (suffix_name))
                datatypes.add(datatype)
            if len(datatypes) != 1:
                raise ValueError("The Pyomo NL file writer found multiple active export suffix components with name %s "\
                                 "with different datatypes. A single datatype must be declared." % (suffix_name))
            if suffix_name == "dual":
                # The NL file format has a special section for dual initializations
                continue
            float_tag = 0
            if datatypes.pop() == Suffix.FLOAT:
                float_tag = 4
            ################## vars
            s_lines = []
            for suffix in suffixes:
                for var_ID in full_var_list:
                    suffix_value = suffix.getValue(Vars_dict[var_ID])
                    if suffix_value is not None:
                        s_lines.append(suffix_line.format(self_ampl_var_id[var_ID],suffix_value))
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write(suffix_header_line.format(var_tag | float_tag, len_s_lines,suffix_name))
                OUTPUT.writelines(s_lines)
                    
            ################## constraints
            s_lines = []
            for suffix in suffixes:
                for con_ID, (con_component, wrapped_ampl_repn) in iteritems(Constraints_dict):
                    suffix_value = suffix.getValue(con_component)
                    if suffix_value is not None:
                        s_lines.append(suffix_line.format(self_ampl_con_id[con_ID],suffix_value))
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write(suffix_header_line.format(con_tag | float_tag, len_s_lines,suffix_name))
                OUTPUT.writelines(s_lines)

            ################## objectives
            s_lines = []
            for suffix in suffixes:
                for obj_ID, (obj_component, wrapped_ampl_repn) in iteritems(Objectives_dict):
                    suffix_value = suffix.getValue(obj_component)
                    if suffix_value is not None:
                        s_lines.append(suffix_line.format(self_ampl_obj_id[obj_ID],suffix_value))
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write(suffix_header_line.format(obj_tag | float_tag, len_s_lines,suffix_name))
                OUTPUT.writelines(s_lines)

            ################## problems (in this case the one problem)
            suffix_value = None
            for suffix in suffixes:
                suffix_value = suffix.getValue(model)
                if suffix_value is not None:
                    break
            if suffix_value is not None:
                OUTPUT.write(suffix_header_line.format(prob_tag | float_tag, 1, suffix_name))
                OUTPUT.write(suffix_line.format(0,suffix_value))
        del modelSOS

        #
        # "C" lines
        #
        rowfilename = None
        if OUTPUT.name.endswith('.nl'):
            rowfilename = OUTPUT.name.replace('.nl','.row')
        else:
            rowfilename = OUTPUT.name+'.row'
        if symbolic_solver_labels is True:
            rowf = open(rowfilename,'w')

        cu = [0 for i in xrange(len(full_var_list))]
        for con_ID in nonlin_con_order_list:
            con_data, wrapped_ampl_repn = Constraints_dict[con_ID]
            row_id = self_ampl_con_id[con_ID]
            OUTPUT.write( "C%d\n"%(row_id) )
            if symbolic_solver_labels is True:
                rowf.write( name_labeler(con_data)+"\n" ) 
            self._print_nonlinear_terms_NL(wrapped_ampl_repn.repn._nonlinear_expr)

            for var_ID in set(wrapped_ampl_repn._linear_vars).union(wrapped_ampl_repn._nonlinear_vars):
                cu[self_ampl_var_id[var_ID]] += 1

        for con_ID in lin_con_order_list:
            con_data, wrapped_ampl_repn = Constraints_dict[con_ID]
            row_id = self_ampl_con_id[con_ID]
            con_vars = set(wrapped_ampl_repn._linear_vars)
            for var_ID in con_vars:
                cu[self_ampl_var_id[var_ID]] += 1
            OUTPUT.write( "C%d\n"%(row_id) )
            if symbolic_solver_labels is True:
                rowf.write( name_labeler(con_data)+"\n" )
            OUTPUT.write( "n0\n" )

        if show_section_timing:
            subsection_timer.report("Write NL header and suffix lines")
            subsection_timer.reset()

        #
        # "O" lines
        #
        for obj_ID, (obj, wrapped_ampl_repn) in iteritems(Objectives_dict):

            k = 0
            if not obj.is_minimizing():
                k = 1

            OUTPUT.write( "O%d %d\n"%(self_ampl_obj_id[obj_ID], k) )
            if symbolic_solver_labels is True:
                rowf.write(name_labeler(obj)+"\n")

            if wrapped_ampl_repn.repn.is_linear():
                OUTPUT.write( "n{0!r}\n".format(wrapped_ampl_repn.repn._constant) )
            else:
                if wrapped_ampl_repn.repn._constant != 0.0:
                    # +
                    OUTPUT.write( "o0\nn{0!r}\n".format(wrapped_ampl_repn.repn._constant) )
                self._print_nonlinear_terms_NL(wrapped_ampl_repn.repn._nonlinear_expr)

        if symbolic_solver_labels is True:
            rowf.close()
        del name_labeler

        if show_section_timing:
            subsection_timer.report("Write objective expression")
            subsection_timer.reset()

        #
        # "d" lines
        #
        # dual initialization
        if 'dual' in suffix_dict:
            s_lines = [] 
            for dual_suffix in suffix_dict['dual']:
                for con_ID, (con_component, wrapped_ampl_repn) in iteritems(Constraints_dict):
                    suffix_value = dual_suffix.getValue(con_component)
                    if suffix_value is not None:
                        s_lines.append(suffix_line.format(self_ampl_con_id[con_ID],suffix_value))
            len_s_lines = len(s_lines)
            if len_s_lines > 0:
                OUTPUT.write( "d%d\n"%(len_s_lines) )
                OUTPUT.writelines(s_lines)
            

        #
        # "x" lines
        #
        # variable initialization
        var_bound_list = []
        x_init_list = []
        for ampl_var_id, var_ID in enumerate(full_var_list):
            var = Vars_dict[var_ID]
            if var.value is not None:
                x_init_list.append("{0} {1!r}\n".format(ampl_var_id,var.value))
            if var.fixed:
                if not output_fixed_variable_bounds:
                    raise ValueError("Encountered a fixed variable (%s) inside an active objective "
                                     "or constraint expression on model %s, which is usually indicative of "
                                     "a preprocessing error. Use the IO-option 'output_fixed_variable_bounds=True' "
                                     "to suppress this error and fix the variable by overwriting its bounds in "
                                     "the NL file."
                                     % (var.cname(True),model.cname(True),))
                if var.value is None:
                    raise ValueError("Variable cannot be fixed to a value of None.")
                L = var.value
                U = var.value
            else:
                L = var.lb
                U = var.ub
            if L is not None:
                Lv = value(L)
                if U is not None:
                    Uv = value(U)
                    if Lv == Uv:
                        var_bound_list.append("4 {0!r}\n".format(Lv))
                    else:
                        var_bound_list.append("0 {0!r} {1!r}\n".format(Lv, Uv))
                else:
                    var_bound_list.append("2 {0!r}\n".format(Lv))
            elif U is not None:
                var_bound_list.append("1 {0!r}\n".format(value(U)))
            else:
                var_bound_list.append("3\n")


        OUTPUT.write( "x%d\n"%(len(x_init_list)) )
        OUTPUT.writelines(x_init_list)
        del x_init_list
        
        if show_section_timing:
            subsection_timer.report("Write initializations")
            subsection_timer.reset()

        #
        # "r" lines
        #
        OUTPUT.write( "r\n" )
        # *NOTE: This iteration follows the assignment of the ampl_con_id
        OUTPUT.writelines(constraint_bounds_dict[con_ID] for con_ID in itertools.chain(nonlin_con_order_list, lin_con_order_list))

        if show_section_timing:
            subsection_timer.report("Write constraint bounds")
            subsection_timer.reset()

        #
        # "b" lines
        #
        OUTPUT.write( "b\n" )
        OUTPUT.writelines(var_bound_list)
        del var_bound_list

        if show_section_timing:
            subsection_timer.report("Write variable bounds")
            subsection_timer.reset()

        #
        # "k" lines
        #
        ktot = 0
        n1 = len(full_var_list) - 1
        OUTPUT.write( "k%d\n"%(n1) )
        ktot = 0
        for i in xrange(n1):
            ktot += cu[i]
            OUTPUT.write( "%d\n"%(ktot) )
        del cu

        if show_section_timing:
            subsection_timer.report("Write k lines")
            subsection_timer.reset()

        #
        # "J" lines
        # 
        for nc, con_ID in enumerate(itertools.chain(nonlin_con_order_list,lin_con_order_list)):
            con_data, wrapped_ampl_repn = Constraints_dict[con_ID]
            num_nonlinear_vars = len(wrapped_ampl_repn._nonlinear_vars)
            num_linear_vars = len(wrapped_ampl_repn._linear_vars)
            if num_nonlinear_vars == 0:
                if num_linear_vars > 0:
                    linear_dict = dict((var_ID, coef) for var_ID, coef in zip(wrapped_ampl_repn._linear_vars, wrapped_ampl_repn.repn._linear_terms_coef))
                    OUTPUT.write( "J%d %d\n"%(nc, num_linear_vars) )
                    OUTPUT.writelines( "{0} {1!r}\n".format(self_ampl_var_id[con_var], linear_dict[con_var]) for con_var in sorted(linear_dict.keys()) )
            elif num_linear_vars == 0:
                nl_con_vars = sorted(wrapped_ampl_repn._nonlinear_vars)
                OUTPUT.write( "J%d %d\n"%(nc, num_nonlinear_vars) )
                OUTPUT.writelines( "%d 0\n"%(self_ampl_var_id[con_var]) for con_var in nl_con_vars )
            else:
                con_vars = set(wrapped_ampl_repn._nonlinear_vars)
                nl_con_vars = sorted(con_vars.difference(wrapped_ampl_repn._linear_vars))
                con_vars.update(wrapped_ampl_repn._linear_vars)
                linear_dict = dict((var_ID, coef) for var_ID, coef in zip(wrapped_ampl_repn._linear_vars, wrapped_ampl_repn.repn._linear_terms_coef))
                OUTPUT.write( "J%d %d\n"%(nc, len(con_vars)) )
                OUTPUT.writelines( "{0} {1!r}\n".format(self_ampl_var_id[con_var], linear_dict[con_var]) for con_var in sorted(linear_dict.keys()) )
                OUTPUT.writelines( "%d 0\n"%(self_ampl_var_id[con_var]) for con_var in nl_con_vars )


        if show_section_timing:
            subsection_timer.report("Write J lines")
            subsection_timer.reset()

        #
        # "G" lines
        #
        for obj_ID, (obj, wrapped_ampl_repn) in iteritems(Objectives_dict):
        
            grad_entries = {}
            for idx, obj_var in enumerate(wrapped_ampl_repn._linear_vars):
                grad_entries[self_ampl_var_id[obj_var]] = wrapped_ampl_repn.repn._linear_terms_coef[idx]
            for obj_var in wrapped_ampl_repn._nonlinear_vars:
                if obj_var not in wrapped_ampl_repn._linear_vars:
                    grad_entries[self_ampl_var_id[obj_var]] = 0
            len_ge = len(grad_entries)
            if len_ge > 0:
                OUTPUT.write( "G%d %d\n"%(self_ampl_obj_id[obj_ID], len_ge) )
                for var_ID in sorted(grad_entries.keys()):
                    OUTPUT.write( "{0} {1!r}\n".format(var_ID, grad_entries[var_ID]) )

        if show_section_timing:
            subsection_timer.report("Write G lines")
            subsection_timer.reset()
            overall_timer.report("Total time")

        return symbol_map


# Switch from Python to C generate_ampl_repn function when possible
#try:
#    py_generate_ampl_repn = generate_ampl_repn
#    from cAmpl import generate_ampl_repn
#except ImportError:
#    del py_generate_ampl_repn

# Alternative: import C implementation under another name for testing
#from cAmpl import generate_ampl_repn as cgar
#__all__.append('cgar')



import ast

from coopr.pyomo.plugins.check.checker import IterativeTreeChecker
from coopr.pyomo.plugins.check.function import FunctionTrackerHook


class _ModelRuleChecker(IterativeTreeChecker):

    FunctionTrackerHook()

    def check(self, runner, script, info):
        if isinstance(info, ast.Call):
            if info.func.id in ['Objective', 'Constraint', 'Var', 'Param', 'Set']:
                for keyword in info.keywords:
                    if keyword.arg == 'rule':
                        if isinstance(keyword.value, ast.Name):
                            funcname = keyword.value.id
                            if funcname in script.functionDefs:
                                funcdef = script.functionDefs[funcname]
                                self.checkBody(funcdef)

    def checkBody(self, funcdef):
        pass



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2012 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

# the purpose of this file is to collect all utility methods that compute
# attributes of blocks, based on their contents. 

__all__ = ['has_discrete_variables']

from six import itervalues, iteritems
from coopr.pyomo.base import Var
from coopr.pyomo.base.block import active_subcomponents_data_generator


def has_discrete_variables(block):
    # Return True if there is a discrete variable in this block or any
    # sub-block.
    for block in block.all_blocks():
        for vardata in active_subcomponents_data_generator(block, Var):
            if not var.is_continuous():
                return True
    return False



import ast

from coopr.pyomo.plugins.check.checker import IterativeTreeChecker
from coopr.pyomo.plugins.check.model import ModelTrackerHook


class ArrayValue(IterativeTreeChecker):

    ModelTrackerHook()

    varArrays = {}

    def checkerDoc(self):
        return """\
        Assigning a value to an array of variables does nothing.
        """

    def checkVarArray(self, script, node):
        """Check for the creation of a new VarArray; store name if created"""

        if isinstance(node.value, ast.Call):
            if isinstance(node.value.func, ast.Name):
                if node.value.func.id == 'Var':
                    if len(node.value.args) > 0:
                        for target in node.targets:
                            if isinstance(target, ast.Attribute):
                                if isinstance(target.value, ast.Name):
                                    if target.value.id in script.modelVars:
                                        if target.value.id not in self.varArrays:
                                            self.varArrays[target.value.id] = []
                                        self.varArrays[target.value.id].append(target.attr)

    def checkArrayValue(self, script, node):
        for target in node.targets:
            if isinstance(target, ast.Attribute):
                if isinstance(target.value, ast.Attribute):
                    if isinstance(target.value.value, ast.Name):
                        if target.value.value.id in script.modelVars:
                            if target.value.value.id in self.varArrays:
                                if target.value.attr in self.varArrays[target.value.value.id]:
                                    if target.attr == 'value':
                                        self.problem("Assigning value to variable array {0}.{1}".format(target.value.value.id, target.value.attr), lineno = node.lineno)

    def check(self, runner, script, info):
        if isinstance(info, ast.Assign):
            self.checkVarArray(script, info)
            self.checkArrayValue(script, info)



import sys
import random
import os.path
import coopr.pyomo.scripting.convert

random.seed(2384792387)

nsets = [i*1000 for i in range(1,8,4)]
nelts = [i*1000 for i in range(1,61,10)]
seeds = [random.getrandbits(32) for i in range(10)]

for seed in seeds:
    random.seed(seed)

    for m in nsets:
        for n in nelts:
            fname = 'scover_%d_%d_%d' % (n, m, seed)
            print 'fname',fname
            coopr.pyomo.scripting.convert.pyomo2lp(args=['--model-options','n=%d m=%d seed=%d type=fixed_element_coverage rho=0.1' % (n,m,seed), '--save-model','%s.lp' % fname, os.path.abspath('../sc.py')])
            
    



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________


__all__ = ['items', 'display_items', 'display_name']

from coopr.pyomo.base.sets import Set
from coopr.pyomo.base.rangeset import RangeSet
from coopr.pyomo.base.param import Param
from coopr.pyomo.base.var import Var
from coopr.pyomo.base.expression import Expression
from coopr.pyomo.base.objective import Objective
from coopr.pyomo.base.constraint import Constraint
from coopr.pyomo.base.sos import SOSConstraint

items = [ Set, RangeSet, Param, Var, Expression, \
        Objective, Constraint, SOSConstraint ]

display_items = [ Var, Objective, Constraint]
display_name = {Var:"Variables", Objective:"Objectives", Constraint:"Constraints"}



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import os.path
import re
try:
    import lxml.etree.ElementTree as ET
except:
    import xml.etree.ElementTree as ET

from coopr.core.plugin import alias

from coopr.pyomo.base.param import Param
from coopr.pyomo.data.TableData import TableData


class XMLTable(TableData):

    alias("xml", "Manage IO with tables in XML files.")

    def __init__(self):
        TableData.__init__(self)

    def open(self):
        if self.filename is None:
            raise IOError("No filename specified")

    def close(self):
        pass

    def read(self):
        if not os.path.exists(self.filename):
            raise IOError("Cannot find file '%s'" % self.filename)
        #
        tree = ET.parse(self.filename)
        if not self.options.query is None:
            if self.options.query[0] == '"' or self.options.query[0] == "'":
                self.options.query = self.options.query[1:-1]
            parents = [parent for parent in tree.findall(self.options.query)]
        else:    
            parents = [parent for parent in tree.getroot()]
        tmp=[]
        labels = []
        for parent in parents:
            if len(tmp) == 0:
                for child in parent:
                    labels.append(child.tag)
                tmp.append(labels)
            row = {}
            for child in parent:
                if child.text is None:
                    row[child.tag] = child.get('value')
                else:
                    row[child.tag] = child.text
            tmp.append( [row.get(label,'.') for label in labels] )
        #
        if len(tmp) == 0:
            raise IOError("Empty *.xml file")
        elif len(tmp) == 1:
            if not self.options.param is None:
                if type(self.options.param) in (list, tuple):
                    p = self.options.param[0]
                else:
                    p = self.options.param
                if isinstance(p, Param):
                    self.options.model = p._model()
                    p = p.name
                self._info = ["param",p,":=",tmp[0][0]]
            elif len(self.options.symbol_map) == 1:
                self._info = ["param",self.options.symbol_map[self.options.symbol_map.keys()[0]],":=",tmp[0][0]]
            else:
                raise IOError("Data looks like a parameter, but multiple parameter names have been specified: %s" % str(self.options.symbol_map))
        else:
            self._set_data(tmp[0], tmp[1:])
        return True

    def write(self, data):
        if self.options.set is None and self.options.param is None:
            raise IOError("Unspecified model component")
        root = ET.Element('table')
        table = self.get_table()
        labels = table[0]
        for i in range(len(labels)):
            labels[i] = str(labels[i])
        for trow in table[1:]:
            row = ET.SubElement(root, 'row')
            for i in range(len(labels)):
                data = ET.SubElement(row, labels[i])
                data.set('value', str(trow[i]))
        #
        tree = ET.ElementTree(root)
        tree.write(self.filename)
        return True




#
# Unit Tests for nontrivial Bounds (_SumExpression, _ProductExpression)
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
from coopr.pyomo import *
from nose.tools import nottest

class Test(unittest.TestCase):

    #Test constraint bounds
    def test_constr_lower(self):
        model = AbstractModel()
        model.A = Param(default=2.0, mutable=True)
        model.B = Param(default=1.5, mutable=True)
        model.C = Param(default=2.5, mutable=True)
        model.X = Var()

        def constr_rule(model):
            return (model.A*(model.B+model.C),model.X)
        model.constr = Constraint(rule=constr_rule)

        instance = model.create()
        self.assertEqual(instance.constr.lower(),8.0)

    def test_constr_upper(self):
        model = AbstractModel()
        model.A = Param(default=2.0, mutable=True)
        model.B = Param(default=1.5, mutable=True)
        model.C = Param(default=2.5, mutable=True)
        model.X = Var()

        def constr_rule(model):
            return (model.X,model.A*(model.B+model.C))
        model.constr = Constraint(rule=constr_rule)

        instance = model.create()

        self.assertEqual(instance.constr.upper(),8.0)

    def test_constr_both(self):
        model = AbstractModel()
        model.A = Param(default=2.0, mutable=True)
        model.B = Param(default=1.5, mutable=True)
        model.C = Param(default=2.5, mutable=True)
        model.X = Var()

        def constr_rule(model):
            return (model.A*(model.B-model.C),model.X,model.A*(model.B+model.C))
        model.constr = Constraint(rule=constr_rule)

        instance = model.create()

        self.assertEqual(instance.constr.lower(),-2.0)
        self.assertEqual(instance.constr.upper(),8.0)


    #Test variable bounds
    #JPW: Disabled until we are convinced that we want to support complex parametric expressions for variable bounds.
    def test_var_bounds(self):
        model = AbstractModel()
        model.A = Param(default=2.0, mutable=True)
        model.B = Param(default=1.5, mutable=True)
        model.C = Param(default=2.5)

        def X_bounds_rule(model):
            return (model.A*(model.B-model.C),model.A*(model.B+model.C))
        model.X = Var(bounds=X_bounds_rule)

        instance = model.create()

        self.assertEqual(instance.X.lb,-2.0)
        self.assertEqual(instance.X.ub,8.0)
   

if __name__ == "__main__":
    unittest.main()




from coopr.pyomo.check.checker import *
from coopr.pyomo.check.runner import *
from coopr.pyomo.check.script import *
from coopr.pyomo.check.hooks import *

# Modules
__all__ = []

# Checker classes
__all__.extend(['IModelChecker'])
#__all__.extend(['ImmediateDataChecker', 'IterativeDataChecker'])
#__all__.extend(['ImmediateTreeChecker', 'IterativeTreeChecker'])

# Other builtins
__all__.extend(['ModelCheckRunner', 'ModelScript'])

# Hooks
__all__.extend(['IPreCheckHook', 'IPostCheckHook'])



from distutils.core import setup, Extension

import os
import fnmatch

sourceFiles = []
for root, dirnames, filenames in os.walk('.'):
    for filename in fnmatch.filter(filenames, "*.c"):
        sourceFiles.append(os.path.join(root, filename))

# Enable for debug only
eca = ['-Werror', '-O3']
cAmpl = Extension('cAmpl', sources = sourceFiles, extra_compile_args = eca)

setup( name = 'cAmpl',
        version = '0.1',
        description = 'C AMPL representation generator',
        ext_modules = [cAmpl] )



"""
Create the diet.sqlite file with all the appropriate data.
"""

import sqlite3

conn = sqlite3.connect('diet.sqlite')

c = conn.cursor()

for table in ['Amount', 'Nutr', 'Food']:
    c.execute('DROP TABLE IF EXISTS ' + table)
conn.commit()

c.execute('''
CREATE TABLE Food (
    FOOD text not null,
    cost float not null,
    f_min float,
    f_max float,
    PRIMARY KEY (FOOD)
)
''')
conn.commit()

Food_data = [
    ("Quarter Pounder w Cheese", 1.84, None, None),
    ("McLean Deluxe w Cheese", 2.19, None, None),
    ("Big Mac", 1.84, None, None),
    ("Filet-O-Fish", 1.44, None, None),
    ("McGrilled Chicken", 2.29, None, None),
    ("Fries, small", 0.77, None, None),
    ("Sausage McMuffin", 1.29, None, None),
    ("1% Lowfat Milk", 0.60, None, None),
    ("Orange Juice", 0.72, None, None)
]
for row in Food_data:
    c.execute('''INSERT INTO Food VALUES (?,?,?,?)''', row)
conn.commit()

c.execute('''
CREATE TABLE Nutr (
    NUTR text not null,
    n_min float,
    n_max float,
    PRIMARY KEY (NUTR)
)
''')

Nutr_data = [
    ("Cal", 2000.0, None),
    ("Carbo", 350.0, 375.0),
    ("Protein", 55.0, None),
    ("VitA", 100.0, None),
    ("VitC", 100.0, None),
    ("Calc", 100.0, None),
    ("Iron", 100.0, None)
]
for row in Nutr_data:
    c.execute('''INSERT INTO Nutr VALUES (?,?,?)''', row)
conn.commit()

c.execute('''
CREATE TABLE Amount (
NUTR text not null,
FOOD varchar not null,
amt float not null,
PRIMARY KEY (NUTR, FOOD),
FOREIGN KEY (NUTR) REFERENCES Nutr (NUTR),
FOREIGN KEY (FOOD) REFERENCES Food (FOOD)
)
''')
conn.commit()

Amount_data = [
    ('Cal','Quarter Pounder w Cheese','510'),
    ('Carbo','Quarter Pounder w Cheese','34'),
    ('Protein','Quarter Pounder w Cheese','28'),
    ('VitA','Quarter Pounder w Cheese','15'),
    ('VitC','Quarter Pounder w Cheese','6'),
    ('Calc','Quarter Pounder w Cheese','30'),
    ('Iron','Quarter Pounder w Cheese','20'),
    ('Cal','McLean Deluxe w Cheese','370'),
    ('Carbo','McLean Deluxe w Cheese','35'),
    ('Protein','McLean Deluxe w Cheese','24'),
    ('VitA','McLean Deluxe w Cheese','15'),
    ('VitC','McLean Deluxe w Cheese','10'),
    ('Calc','McLean Deluxe w Cheese','20'),
    ('Iron','McLean Deluxe w Cheese','20'),
    ('Cal','Big Mac','500'),
    ('Carbo','Big Mac','42'),
    ('Protein','Big Mac','25'),
    ('VitA','Big Mac','6'),
    ('VitC','Big Mac','2'),
    ('Calc','Big Mac','25'),
    ('Iron','Big Mac','20'),
    ('Cal','Filet-O-Fish','370'),
    ('Carbo','Filet-O-Fish','38'),
    ('Protein','Filet-O-Fish','14'),
    ('VitA','Filet-O-Fish','2'),
    ('VitC','Filet-O-Fish','0'),
    ('Calc','Filet-O-Fish','15'),
    ('Iron','Filet-O-Fish','10'),
    ('Cal','McGrilled Chicken','400'),
    ('Carbo','McGrilled Chicken','42'),
    ('Protein','McGrilled Chicken','31'),
    ('VitA','McGrilled Chicken','8'),
    ('VitC','McGrilled Chicken','15'),
    ('Calc','McGrilled Chicken','15'),
    ('Iron','McGrilled Chicken','8'),
    ('Cal','Fries, small','220'),
    ('Carbo','Fries, small','26'),
    ('Protein','Fries, small','3'),
    ('VitA','Fries, small','0'),
    ('VitC','Fries, small','15'),
    ('Calc','Fries, small','0'),
    ('Iron','Fries, small','2'),
    ('Cal','Sausage McMuffin','345'),
    ('Carbo','Sausage McMuffin','27'),
    ('Protein','Sausage McMuffin','15'),
    ('VitA','Sausage McMuffin','4'),
    ('VitC','Sausage McMuffin','0'),
    ('Calc','Sausage McMuffin','20'),
    ('Iron','Sausage McMuffin','15'),
    ('Cal','1% Lowfat Milk','110'),
    ('Carbo','1% Lowfat Milk','12'),
    ('Protein','1% Lowfat Milk','9'),
    ('VitA','1% Lowfat Milk','10'),
    ('VitC','1% Lowfat Milk','4'),
    ('Calc','1% Lowfat Milk','30'),
    ('Iron','1% Lowfat Milk','0'),
    ('Cal','Orange Juice','80'),
    ('Carbo','Orange Juice','20'),
    ('Protein','Orange Juice','1'),
    ('VitA','Orange Juice','2'),
    ('VitC','Orange Juice','120'),
    ('Calc','Orange Juice','2'),
    ('Iron','Orange Juice','2')
]
for row in Amount_data:
    c.execute('''INSERT INTO Amount VALUES (?,?,?)''', row)
conn.commit()
